from typing import Callable
import argparse
import shutil
import stat
import os
import os.path as osp
from glob import glob
import platform
import subprocess


parser = argparse.ArgumentParser("clear cache")
parser.add_argument(
    "-e",
    "--expand_cache_path",
    default=[],
    nargs="+",
    help="clear default path, and expand cache path ",
)
args = parser.parse_args()

pip_cache = osp.join(osp.expanduser("~"), ".cache/pip")
conda_cache = osp.join(os.popen("conda info --root").read().strip(), "pkgs")
trash = osp.join(osp.expanduser("~"), ".local/share/Trash")

if platform.system() == "Windows":
    args.roots = [
        osp.join(osp.expanduser("~"), "AppData/Local/pip/cache"),
        conda_cache,
    ] + args.expand_cache_path
elif platform.system() == "Linux":
    args.roots = [
        osp.join(osp.expanduser("~"), ".cache/pip"),
        conda_cache,
        trash,
    ] + args.expand_cache_path


def remove_readonly(
    func: Callable, path: str, _
) -> None:  # 错误回调函数，改变只读属性位，重新删除
    "Clear the readonly bit and reattempt the removal"
    os.chmod(path, stat.S_IWRITE)
    func(path)


def clear_cache() -> None:
    for root in args.roots:
        if osp.exists(root):
            total_size = 0
            for f in glob(osp.join(root, "**"), recursive=True):
                if osp.isfile(f):
                    total_size += osp.getsize(f)
            print(f"{root}: {total_size/1024/1024:.2f} MB")

    command = input("clear cache? (y/n):")
    if command == "y":
        os.system("pip cache purge")
        os.system("conda clean --all")
        for root in args.roots:
            if osp.exists(root):
                try:
                    shutil.rmtree(
                        root, onerror=remove_readonly
                    )  # 如报错，可能要管理员权限运行此程序
                    os.mkdir(root)
                except:
                    for f in glob(osp.join(root, "**"), recursive=True):
                        if osp.isfile(f):
                            try:
                                os.remove(f)
                            except PermissionError:
                                print(f"{f} remove failed for PermissionError")

        subprocess.run(
            "pip uninstall rlmc -y",
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )


if __name__ == "__main__":
    clear_cache()



# -*- coding:utf-8 -*-
# @Time    : 2023/1/13 23:20


from datasets import *

def test(model, enc_input, start_symbol):
    # Starting Reference: http://nlp.seas.harvard.edu/2018/04/03/attention.html#greedy-decoding
    enc_outputs, enc_self_attns = model.Encoder(enc_input)
    dec_input = torch.zeros(1, tgt_len).type_as(enc_input.data)
    next_symbol = start_symbol
    for i in range(0, tgt_len):
        dec_input[0][i] = next_symbol
        dec_outputs, _, _ = model.Decoder(dec_input, enc_input, enc_outputs)
        projected = model.projection(dec_outputs)
        prob = projected.squeeze(0).max(dim=-1, keepdim=False)[1]
        next_word = prob.data[i]
        next_symbol = next_word.item()
    return dec_input

enc_inputs, dec_inputs, dec_outputs = make_data()
loader = Data.DataLoader(MyDataSet(enc_inputs, dec_inputs, dec_outputs), 2, True)
enc_inputs, _, _ = next(iter(loader))
model = torch.load('model.pth')

# predict_dec_input = test(model, enc_inputs[0].view(1, -1).cuda(), start_symbol=tgt_vocab["S"])
# predict, _, _, _ = model(enc_inputs[0].view(1, -1).cuda(), predict_dec_input)
# predict=predict.data.max(1, keepdim=True)[1]


predict_dec_input=['S','P','P','P','P']
predict_dec_input = torch.tensor([[tgt_vocab[symbol] for symbol in predict_dec_input]]).T.cuda()
for i in range(tgt_len):
    predict, _, _, _ = model(enc_inputs[0].view(1, -1).cuda(), predict_dec_input.T)
    
    pre=predict.data.max(1, keepdim=True)[1][i][0]

    if pre != tgt_vocab["E"] and i < tgt_len-2:
        predict_dec_input[i+1][0] = pre
        predict_dec_input[i+2:][0] = tgt_vocab["P"]
    elif pre != tgt_vocab["E"] and i < tgt_len-1:
        predict_dec_input[i+1][0] = pre
    elif pre == tgt_vocab["E"] and i < tgt_len-1:
        predict_dec_input[i+1:][0]= tgt_vocab["E"]
        break


predict=predict_dec_input[1:].cpu()
element_to_add = torch.tensor([[tgt_vocab['E']]])
predict = torch.cat((predict, element_to_add), dim=0)

print([src_idx2word[int(i)] for i in enc_inputs[0]], '->',
      [idx2word[n.item()] for n in predict.squeeze()])


"""
@File    :   __init__.py
@Time    :   2024/06/21 22:38:06
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Any, Callable, Generic, TypeVar

from rlmc.fileop.yaml_op import Yaml
from rlmc.fileop.json_op import Json
from rlmc.fileop.numpy_op import Numpy
from rlmc.fileop.pickle_op import Pickle
from rlmc.fileop.mat_op import Mat

__all__ = ["file_processor"]

T=TypeVar('T')

files_op_dict = {
    "yaml": Yaml,
    "yml": Yaml,
    "json": Json,
    "npy": Numpy,
    "pkl": Pickle,
    "pickle": Pickle,
    "mat": Mat,
}


def file_processor(file_path: str = "", file_ext: str = "") -> Any:
    if file_ext == "":
        file_type = file_path.split(".")[-1]
    else:
        file_type = file_ext
    return files_op_dict.get(file_type)(file_path)


if __name__ == "__main__":
    yamlobj = file_processor("rlmc/configs/script_paths.yaml")
    yaml_content = yamlobj.data
    # yaml_content = yamlobj.read("rlmc/configs/script_paths.yaml")
    print(yaml_content)



"""
@File    :   gittool.py
@Time    :   2024/06/26 22:08:37
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import List, Dict, Union
import os
import os.path as osp

import git


class Git:
    """
    Not used, assist to remember the git command.
    """

    def __init__(self, local_repo_path=None):
        self.local_repo_path = local_repo_path
        self.local_repo = None
        self.remote_url = None
        self.remote_branchs = None
        if local_repo_path != None:
            if osp.isdir(osp.join(self.local_repo_path, ".git")):
                self.open_local_repo()

    def init_local_repo(self, remote_url: str) -> None:
        """
        Initialize a local repository.
        Args:
            remote_url (str): https://github.com/xxxx/xxxx.git
        """
        self.local_repo = git.Repo.init(self.local_repo_path)
        origin = self.local_repo.create_remote("origin", remote_url)
        origin.fetch()
        self.get_remote_info()


    def open_local_repo(self) -> None:
        self.local_repo = git.Repo(self.local_repo_path)
        self.get_remote_info()

    def get_config(self) -> List:
        return self.local_repo.git.config("--list").split("\n")

    def set_config(self, key="https.proxy", value="https://127.0.0.1:7890"):
        self.local_repo.git.config("--global", key, value)

    def get_repo_status(self) -> str:
        return self.local_repo.git.status()

    def get_file_status(self, file_path) -> str:
        return self.local_repo.git.status(file_path)

    def git_stash(self, type="save") -> None:
        if type == "save":
            self.local_repo.git.stash()
        elif type =="pop":
            self.local_repo.git.stash("pop")
        elif type == "view":
            self.local_repo.git.stash("list")


    def get_repo_history(self) -> str:
        return self.local_repo.git.log()

    def get_file_history(self, file_path) -> str:
        return self.local_repo.git.log("--follow", "--", file_path)

    def get_commits_history(self):
        commit_history = list(self.local_repo.iter_commits())
        return commit_history

    def get_local_repo_path(self) -> str:
        return self.local_repo.working_dir

    def get_remote_info(self) -> git.remote.Remote:  

        self.remote = self.local_repo.remote()  # origin
        self.remote_url = self.remote.url
        self.remote_branchs = self.remote.refs
        return self.remote

    def get_current_branch(self) -> str:
        return self.local_repo.active_branch  # master,main...

    def create_branch(self, branch_name) -> None:
        self.local_repo.create_head(branch_name)

    def delete_branch(self, branch_name) -> None:
        self.local_repo.delete_head(branch_name)

    def switch_branch(self, branch_name) -> None:
        return self.local_repo.active_branch # master,main...

    def create_branch(self, branch_name) -> None:  
        self.local_repo.create_head(branch_name)

    def delete_branch(self, branch_name) -> None: 
        self.local_repo.delete_head(branch_name)

    def switch_branch(self, branch_name) -> List: 
        self.local_repo.git.checkout(branch_name)

    def get_all_branchs(self) -> List:
        return self.local_repo.git.branch("-a").split("\n")

    def get_all_tags(self) -> List:
        """获取标签列表"""
        return self.local_repo.git.tag().split("\n")

    def get_latest_author_info(self) -> str:
        return self.local_repo.head.commit.author

    def get_diff_between_commits(
        self, commit_sha_1, commit_sha_2
    ) -> List[git.diff.Diff]:
        """获取两个提交之间的差异   git.diff.Diff可print"""

    def get_diff_between_commits(self, commit_sha_1, commit_sha_2) -> List[git.diff.Diff]: 
        """获取两个提交之间的差异   git.diff.Diff可print"""   
        commit_1 = self.local_repo.commit(commit_sha_1)
        commit_2 = self.local_repo.commit(commit_sha_2)
        return commit_1.diff(commit_2)


    def get_file_content_in_commit(self, file_path, commit_sha) -> str:  

        """获取某个文件在某个提交中的内容 要用相对路径例如rltests/test.py"""
        commit = self.local_repo.commit(commit_sha)
        return commit.tree[file_path].data_stream.read().decode("utf-8")

    def get_untracked_files(self) -> List[str]:
        """获取未跟踪的文件列表"""
        return self.local_repo.untracked_files

    def cancel_uncommit_changes(self):  #
        """撤销未提交的更改"""
        self.local_repo.git.reset("--hard", "HEAD")


    def check_uncommit_changes(self) -> bool:   
        """检查是否有未提交的更改"""
        return self.local_repo.is_dirty()

    def roll_back_to_commit(self, commit_sha):  #
        """回滚到某个提交"""
        commit_to_roll_back = self.local_repo.commit(commit_sha)
        self.local_repo.git.reset("--hard", commit_to_roll_back)

    def git_add(self, file="all"):
        if file == "all":
            self.local_repo.git.add(all=True)
        else:
            self.local_repo.git.add(file)

    def git_commit(self, commit_msg="update some modules"):
        self.local_repo.git.commit("-m", commit_msg)

    def git_merge(self, branch_name):  # "--allow-unrelated-histories"
        self.local_repo.git.merge(branch_name)

    def git_push_repo(self, commit_msg="update some modules"):  #
        self.git_add()
        self.git_commit(commit_msg)
        self.local_repo.remotes.origin.push()


    def git_pull_repo(self, commit_msg="update some modules"):  #
        self.git_add()
        self.git_commit(commit_msg)
        self.local_repo.remotes.origin.pull()

    def git_clone(self, remote_repo_url):
        git.Repo.clone_from(remote_repo_url, self.local_repo_path)
        self.open_local_repo()
        self.get_remote_info()


    def git_push_one_file(self, file_path, commit_msg="update some modules"):  #
        self.git_add(file_path)
        self.git_commit(commit_msg)
        self.local_repo.remotes.origin.push()


if __name__ == "__main__":
    local_repo_path = r"test_git"
    remote_repo_url = "https://github.com/RayLam2022/rlmc.git"

    gt = Git(local_repo_path)

    # gt.git_clone(remote_repo_url)
    # gt.git_pull_repo()
    # gt.create_branch('test_branch1')
    print(gt.get_all_branchs())
    gt.switch_branch("test_branch")
    # x=gt.get_diff_between_commits('2a68b8e5f9bcc59e211163d5f22ab0a3fe6bbddc','2a0dc428868e7fbd3a0128876b8ef221a4f4ff23')[1]
    # x=gt.get_file_content_in_commit(r'rltests/test.py','2a0dc428868e7fbd3a0128876b8ef221a4f4ff23')
    #
    # gt.git_pull_repo()
    print(gt.get_repo_status())
    print(gt.get_untracked_files())
    gt.delete_branch("remotes/origin/main")

    gt.git_push_repo("create g_tools")
    # gt.git_push_one_file(r'','modify g_tools')

    #gt.git_clone(remote_repo_url)
    #gt.create_branch('test_branch')
    print(gt.get_all_branchs())
    gt.switch_branch('test_branch')
    #x=gt.get_diff_between_commits('2a68b8e5f9bcc59e211163d5f22ab0a3fe6bbddc','2a0dc428868e7fbd3a0128876b8ef221a4f4ff23')[1]
    #x=gt.get_file_content_in_commit(r'rltests/test.py','2a0dc428868e7fbd3a0128876b8ef221a4f4ff23')
    #gt.git_pull_repo()
    print(gt.get_repo_status())
    gt.git_push_one_file(r'')

    #print(x)




"""
@File    :   predictor.py
@Time    :   2024/06/30 00:24:58
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Union, Dict, List

import torch
import torch.nn as nn


__all__ = [
    "ClsPredictor",
    "SemanticSegmentationPredictor",
    "PosePredictor",
    "ObjDetectPredictor",
    "RegressorPredictor",
    "SequencePredictor",
]


class ClsPredictor(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs


class SemanticSegmentationPredictor(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs
        self.mlp_head = nn.Sequential(
            nn.LayerNorm(self.args.model.hidden_dim),
            nn.Linear(self.args.model.hidden_dim, self.args.model.num_classes),
        )
        self.act_fn = nn.Softmax(dim=1)

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        return self.act_fn(self.mlp_head(x))


class PosePredictor(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs


class ObjDetectPredictor(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs


class ObbPredictor(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs


class RegressorPredictor(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs


class SequencePredictor(torch.nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        self.args = configs



"""
@File    :   semantic_segmentation.py
@Time    :   2024/06/30 00:24:37
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Union, Dict, List

import torch
import torch.nn as nn
from torchvision import transforms
import timm
import segmentation_models_pytorch as smp
from PIL import Image


__all__ = ["SemanticSegmentationModel"]


seg_models = {
    "DeepLabV3": smp.DeepLabV3,
    "DeepLabV3Plus": smp.DeepLabV3Plus,
    "FPN": smp.FPN,
    "Linknet": smp.Linknet,
    "MAnet": smp.MAnet,
    "PAN": smp.PAN,
    "PSPNet": smp.PSPNet,
    "Unet": smp.Unet,
    "UnetPlusPlus": smp.UnetPlusPlus,
}


class SemanticSegmentationModel(nn.Module):
    def __init__(self, configs: Dict) -> None:
        super().__init__()
        # model_ref: https://github.com/qubvel/segmentation_models.pytorch
        self.args = configs
        # self.model_names = timm.list_models("deep",pretrained=self.args.model.pretrained)
        # print(self.model_names)
        # self.backbone=timm.create_model(self.args.model.name, pretrained=self.args.model.pretrained)
        self.model = seg_models[self.args.model.modeltype](
            encoder_name=self.args.model.backbone,
            encoder_weights=self.args.model.encoder_weights,
            in_channels=self.args.model.in_channels,
            classes=self.args.model.num_classes,
            activation=self.args.model.activation,
        )

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        out = self.model(x)
        return out



import re
import copy
from typing import Any, Union, Optional, Generator, Iterable
from typing import Sequence, Tuple, Dict, List, Set
import pprint

__all__ = ["SuDict"]


class SuDict(dict):

    def __init__(self, *args, **kwargs) -> None:
        # 不能在实例中增加其他属性
        for arg in args:
            if not arg:
                continue
            elif isinstance(arg, dict):
                for key, val in arg.items():
                    self[key] = self._hook(val)
            elif isinstance(arg, tuple) and (not isinstance(arg[0], tuple)):
                self[arg[0]] = self._hook(arg[1])
            else:
                for key, val in iter(arg):
                    self[key] = self._hook(val)

        for key, val in kwargs.items():
            self[key] = self._hook(val)

    def __setattr__(self, name, value):
        self[name] = value

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __setitem__(self, name, value):
        super().__setitem__(name, value)

    def __getitem__(self, name):
        # print('getitem',self.__class_getitem__(name))
        return super().__getitem__(name)

    def __missing__(self, name) -> str:
        return ""  # f'missing key:{name}'

    def __delattr__(self, name):
        del self[name]

    def _flatten(
        self,
        data: Dict,
        results: Optional[str] = None,
        temp_sep: str = "_",
        digit_symbol: str = "#",
    ) -> Generator:
        if results is None:
            results = ""
        for key, value in data.items():
            if isinstance(key, int):
                k = f"{digit_symbol}{key}"
            else:
                k = f"{key}"

            if isinstance(value, dict) and value:
                for item in self._flatten(value, f"{results}{temp_sep}{k}"):
                    yield item
            else:
                yield f"{results}{temp_sep}{k}", value

    def create_flat_kv(self, temp_sep: str = "_") -> List[Tuple]:
        data = copy.deepcopy(self)
        flatdic = {k: v for k, v in self._flatten(data, temp_sep=temp_sep)}
        return sorted(flatdic.items(), key=lambda d: len(d[0].split(temp_sep)))

    @classmethod
    def _hook(cls, item: Sequence):
        if isinstance(item, dict):
            return cls(item)
        elif isinstance(item, (list, tuple)):
            return type(item)(cls._hook(elem) for elem in item)
        return item

    def copy(self):
        return copy.copy(self)

    def deepcopy(self):
        return copy.deepcopy(self)

    def __deepcopy__(self, memo):
        other = self.__class__()
        memo[id(self)] = other
        for key, value in self.items():
            other[copy.deepcopy(key, memo)] = copy.deepcopy(value, memo)
        return other

    def setdefault(self, key, default=None):
        if key in self:
            return self[key]
        else:
            self[key] = default
            return default

    def to_dict(self) -> Dict:
        base = {}
        for key, value in self.items():
            if isinstance(value, type(self)):
                base[key] = value.to_dict()
            elif isinstance(value, (list, tuple)):
                base[key] = type(value)(
                    item.to_dict() if isinstance(item, type(self)) else item
                    for item in value
                )
            else:
                base[key] = value
        return base

    def update(self, *args, **kwargs) -> None:
        other = dict()
        if args:
            if len(args) > 1:
                raise TypeError()
            other.update(args[0])
        other.update(kwargs)
        for k, v in other.items():
            if (
                (k not in self)
                or (not isinstance(self[k], dict))
                or (not isinstance(v, dict))
            ):
                self[k] = v
            else:
                self[k].update(v)

    def keystr2keylist(self, key_str:str, sep:str=".", digit_symbol:str="#") -> List:
        lis = key_str.split(sep)
        lis = [int(l[1:]) if l[0] == digit_symbol else l for l in lis]
        return lis

    def search_all_keys(
        self,
        key: Union[str, List],
        flat_kv: List[Tuple],
        is_head: bool = False,
        temp_sep: str = "_",
        digit_symbol: str = "#",
    ) -> List[Tuple]:

        assert isinstance(
            key, (str, list)
        ), "查找的key要是列表形式如[key1,key2...]，或者以分隔符.分隔的节点序列如key1.key2.key3...字符串形式"

        if isinstance(key, list):
            key = [f"{digit_symbol}{l}" if isinstance(l, int) else l for l in key]
            key = temp_sep.join(key)
        else:
            key.replace(".", temp_sep)

        collection = []
        for k in flat_kv:
            s = k[0][1:]

            if key in s:
                if is_head:
                    pattern = f"^{key}{temp_sep}.*|^{key}$"
                else:
                    pattern = f".*{temp_sep}{key}{temp_sep}.*|.*{temp_sep}{key}$"
                # print('pattern:',pattern,'s:',s)
                result = re.match(pattern, s)
                if not isinstance(result, type(None)):
                    collection.append((s.replace(temp_sep, "."), k[1]))

        return collection


if __name__ == "__main__":
    nest_dict = {
        "a": 1,
        "b": {"c": 2, "d": 3, "e": {"f": 4}},
        "g": {"h": 5},
        "i": 6,
        "j": {"k": {7: {"m": 8}}},
        "n": [1, {"o": 1, "p": [1, 2, 3], "q": {"r": {"s": 100}}}, 3, [1, 2, 3], 5],
    }

    d = SuDict(nest_dict)

    d.update(SuDict({6: {"x": 9}}))
    d.update({"s": "c"})
    pprint.pprint(d)
    print(d.j.k[7])
    print(d.j.k)
    print("-----")
    flat_kv = d.create_flat_kv()
    pprint.pprint(flat_kv)
    print(d.search_all_keys([6, "x"], flat_kv, is_head=True))

    # d.s.e=9
    # print(d.s.e)



"""
@File    :   mc.py
@Time    :   2024/06/19 01:50:44
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Any, List, Dict

__all__ = ["MetaList"]


# class MetaList(type):

#     def __new__(meta, name, bases, attrs):
#         _cls = super().__new__(meta, name, bases, attrs)
#         setattr(_cls, "__lshift__", lambda self, other: self.extend(other))
#         for method in dir(meta):
#             try:
#                 met = getattr(meta, method)
#                 if "__" not in method and callable(met):
#                     setattr(_cls, method, met)
#             except:
#                 ...
#                 # print('except',method)
#         return _cls


#     def head_anno(self, sep: str = "nan", is_eq_sep: bool = False) -> List:
#         if is_eq_sep:
#             index = [idx for idx, i in enumerate(self) if i == sep]
#         else:
#             index = [idx for idx, i in enumerate(self) if i != sep]
#         res = [[index[idx], index[idx + 1]] for idx, i in enumerate(index[:-1])]
#         if res[-1][1] != len(self) + 1:
#             res.append([res[-1][1], len(self) + 1])
#         # for i in res:
#         #     print(self[i[0]:i[1]])
#         return res

#     def tail_anno(self, sep: str = "nan", is_eq_sep: bool = False) -> List:
#         if is_eq_sep:
#             ind = [idx for idx, i in enumerate(self) if i == sep]
#         else:
#             ind = [idx for idx, i in enumerate(self) if i != sep]

#         index = [-1]
#         index.extend(ind)

#         res = [[index[idx] + 1, index[idx + 1] + 1] for idx, i in enumerate(index[:-1])]
#         # for i in res:
#         #     print(self[i[0]:i[1]])
#         return res

#     def appendleft(self, elem: Any) -> None:
#         self.insert(0, elem)


class Ext(list):

    def head_anno(self, sep: str = "nan", is_eq_sep: bool = False) -> List:
        if is_eq_sep:
            index = [idx for idx, i in enumerate(self) if i == sep]
        else:
            index = [idx for idx, i in enumerate(self) if i != sep]
        res = [[index[idx], index[idx + 1]] for idx, i in enumerate(index[:-1])]
        if res[-1][1] != len(self) + 1:
            res.append([res[-1][1], len(self) + 1])
        # for i in res:
        #     print(self[i[0]:i[1]])
        return res

    def tail_anno(self, sep: str = "nan", is_eq_sep: bool = False) -> List:
        if is_eq_sep:
            ind = [idx for idx, i in enumerate(self) if i == sep]
        else:
            ind = [idx for idx, i in enumerate(self) if i != sep]

        index = [-1]
        index.extend(ind)

        res = [[index[idx] + 1, index[idx + 1] + 1] for idx, i in enumerate(index[:-1])]
        # for i in res:
        #     print(self[i[0]:i[1]])
        return res

    def appendleft(self, elem: Any) -> None:
        self.insert(0, elem)


class MetaList(type):

    def __new__(meta, name, bases, attrs):

        _cls = super().__new__(meta, name, bases, attrs)
        setattr(_cls, "__lshift__", lambda self, other: self.extend(other))
        for method in Ext.__dict__:

            try:
                et = getattr(Ext, method)
                if "__" not in method and callable(et):
                    setattr(_cls, method, et)
            except:
                ...
                # print('except',method)
        return _cls


def metalist(cls):
    return MetaList(cls.__name__, cls.__bases__, {})


if __name__ == "__main__":

    @metalist
    class B(list): ...

    b = B()

    c = [10, "nan", "nan", 80, 70, 60, "nan", 0, 20, "nan", "nan"]
    c = c[::-1]
    print(c)
    b << c
    b.appendleft(666)
    print(b)
    # index=b.head_anno()
    index = b.tail_anno()

    for i in index:
        print(b[i[0] : i[1]])
    d = B([9, 0, "c", "0"])
    type(d)
    print(type(MetaList))



import argparse
import timeit
from importlib import import_module


parser = argparse.ArgumentParser("cuda_test")
parser.add_argument("-m", "--module", required=True, help="the module to test")
args = parser.parse_args()


__all__ = ["cudatest"]


def torchtest(th) -> None:
    print(th.__version__)
    tensor = th.Tensor([1.0])
    tensor = tensor.cuda()
    print(th.backends.cudnn.is_acceptable(tensor))


def tftest(tf) -> None:

    print(tf.test.is_gpu_available())
    print(tf.test.is_built_with_cuda())

    with tf.device("/cpu:0"):
        cpu_a = tf.random.normal([10000, 1000])
        cpu_b = tf.random.normal([1000, 2000])
        print(cpu_a.device, cpu_b.device)

    with tf.device("/gpu:0"):
        gpu_a = tf.random.normal([10000, 1000])
        gpu_b = tf.random.normal([1000, 2000])
        print(gpu_a.device, gpu_b.device)

    def cpu_run():
        with tf.device("/cpu:0"):
            met = tf.matmul(cpu_a, cpu_b)
        return met

    def gpu_run():
        with tf.device("/gpu:0"):
            met = tf.matmul(gpu_a, gpu_b)
        return met

    # warm up
    cpu_time = timeit.timeit(cpu_run, number=10)
    gpu_time = timeit.timeit(gpu_run, number=10)
    print("warmup:", cpu_time, gpu_time)

    cpu_time = timeit.timeit(cpu_run, number=10)
    gpu_time = timeit.timeit(gpu_run, number=10)
    print("run time:", cpu_time, gpu_time)


def cudatest() -> None:
    assert args.module in ["torch", "tensorflow"], "module参数错误,可选torch,tensorflow"
    if args.module == "torch":
        try:
            th = import_module("torch")
            tv = import_module("torchvision")
            print("torchvision version", tv.__version__)
            torchtest(th)
        except:
            print("torch或torchvision未安装或者存在安装异常")
    elif args.module == "tensorflow":
        try:
            tf = import_module("tensorflow")
            tftest(tf)
        except:
            print("tensorflow未安装,超过测试版本或者存在安装异常")


if __name__ == "__main__":
    cudatest()



from examples_cy import hello_cython,pi_cy
import time

def pi_py(N):
    pi = 0
    for n in range(N):
        pi += (-1) **n / (2*n + 1)
    return 4*pi

if __name__ == "__main__":
    print(hello_cython())
    
    # stime = time.time()
    # for _ in range(100):
    #     out1 = pi_py(1000000)
    # etime = time.time()

    stime2 = time.time()
    for _ in range(100):
        out2 = pi_cy(1000000)
    etime2 = time.time()

    # print("pi_py result {} time cost {} s".format(out1, etime - stime))
    print("pi_cy result {} time cost {} s".format(out2, etime2 - stime2))



"""
@File    :   yaml_op.py
@Time    :   2024/06/21 20:46:02
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

import os
import yaml

from rlmc.fileop.abstract_file import AbstractFile

__all__ = ["Yaml"]


class Yaml(AbstractFile):
    def __init__(
        self, file_path: str = "", mode: str = "r", encoding: str = "utf-8"
    ) -> None:
        self.file_path = file_path
        self.encoding = encoding
        self.mode = mode

        if file_path != "":
            self.data = self.read(file_path)

    def __enter__(self):
        self.file = open(self.file_path, self.mode, encoding=self.encoding)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.file.close()
        if exc_type != None:
            print(exc_type, exc_val, exc_tb)

    def read(self, file_path: str = ""):
        with open(file_path, self.mode, encoding=self.encoding) as f:
            data = yaml.safe_load(f)
        return data

    def write(
        self, data, file_path: str, mode: str = "w", encoding: str = "utf-8"
    ) -> None:
        with open(file_path, mode, encoding=encoding) as f:
            yaml.safe_dump(data, f, allow_unicode=True)


if __name__ == "__main__":
    yamlobj = Yaml("rlmc/configs/script_paths.yaml")
    yaml_content = yamlobj.data
    print(yaml_content)
    nest_dict = {
        "a": 1,
        "b": {"c": 2, "d": 3, "e": {"f": 4}},
        "g": {"h": 5},
        "i": 6,
        "j": {"k": {7: {"m": 8}}},
        "n": [1, {"o": 1, "p": [1, 2, 3], "q": {"r": {"s": 100}}}, 3, [1, 2, 3], 5],
    }

    with Yaml("rlmc/configs/script_paths.yaml") as yl:
        print(yl.data)

    # yamlobj.write(nest_dict, "rlmc/configs/cfg1.yaml")



"""
@File    :   ocr.py
@Time    :   2024/07/09 12:05:48
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from importlib import import_module
import multiprocessing as mp
import os
import sys
import time
import io
import argparse
from typing import Sequence, Union, List, Tuple, Optional, Dict, Any

import cv2
from PIL import ImageGrab
import numpy as np
import pynput
import pyperclip
import easyocr
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM, pipeline


parser = argparse.ArgumentParser("ocr")
parser.add_argument(
    "-l",
    "--language",
    nargs="+",
    default=["ch_sim", "en"],
    help="ch_sim, en, ko, ja and so on",
)
parser.add_argument(
    "-tl",
    "--tolerance",
    type=float,
    default=0.30,
    help="the tolerance to divide the line",
)
parser.add_argument("-i", "--is_translate", action="store_true", help="is_translate")
parser.add_argument(
    "-s",
    "--translate_src_lang",
    default="eng_Latn",
    help="translate zho_Hans, zho_Hant,eng_Latn, jpn_Jpan,yue_Hant,kor_Hang",
)
parser.add_argument(
    "-t",
    "--translate_tgt_lang",
    default="zho_Hans",
    help="translate zho_Hans, zho_Hant,eng_Latn, jpn_Jpan,yue_Hant,kor_Hang",  # zho_Hant繁体
)
parser.add_argument(
    "-m",
    "--translate_model",
    default="facebook/nllb-200-distilled-600M",
    help="translate model",
)
args = parser.parse_args()


signal = 0


def check_bbox(bbox: Sequence) -> bool:
    if bbox[0] < 0 or bbox[1] < 0 or bbox[2] < 0 or bbox[3] < 0:
        return False
    if bbox[0] > bbox[2] or bbox[1] >= bbox[3]:
        return False
    return True


def get_screenshot(bbox: Sequence) -> np.ndarray:
    byteImgIO = io.BytesIO()
    img = ImageGrab.grab(bbox)
    img.save(byteImgIO, format="PNG")
    byteImgIO.seek(0)
    img = cv2.imdecode(np.frombuffer(byteImgIO.read(), np.uint8), cv2.IMREAD_COLOR)
    return img


def on_click(x, y, button, pressed) -> Optional[bool]:
    global bbox
    if not pressed:
        bbox[2:] = [x, y]
        return False
    else:
        bbox[0:2] = [x, y]


def on_press(key: pynput.keyboard.Key) -> Optional[bool]:
    if key == pynput.keyboard.Key.esc:
        global signal
        signal = 1
        return False
    elif hasattr(key, "char"):
        if key.char == "f":
            return False

    # print("Key {0} pressed".format(key))


def ms_listener() -> None:
    listener = pynput.mouse.Listener(on_click=on_click)
    listener.start()
    listener.join()


def kb_listener() -> None:
    listener = pynput.keyboard.Listener(on_press=on_press)
    listener.start()
    listener.join()


def main() -> None:
    global signal
    global bbox
    print("lang: ", args.language)
    reader = easyocr.Reader(args.language)
    if args.is_translate:
        tokenizer = AutoTokenizer.from_pretrained(args.translate_model)
        translate_model = AutoModelForSeq2SeqLM.from_pretrained(args.translate_model)

        translator = pipeline(
            "translation",
            model=translate_model,
            tokenizer=tokenizer,
            src_lang=args.translate_src_lang,
            tgt_lang=args.translate_tgt_lang,
            max_length=512,
            device=0,
        )
        print(
            "translate model is ready, src_lang: ",
            args.translate_src_lang,
            "tgt_lang: ",
            args.translate_tgt_lang,
        )
    print('easeocr is ready, press "f" to select area, press "esc" to exit')
    while True:
        bbox = [-1, -1, -1, -1]
        kb_listener()
        if signal == 1:
            break
        ms_listener()
        # print(bbox)
        if check_bbox(bbox):
            img = get_screenshot(bbox)
            result = reader.readtext(img)
            print(result)
            text = ""
            translated_text = ""
            y_top_compare = 0
            y_bot_compare = 0
            for idx, res in enumerate(result):

                y_top_val, y_bot_val = result[idx][0][0][1], result[idx][0][2][1]
                gap = (y_bot_val - y_top_val) * args.tolerance
                # 如y_top_val大等于y_top_compare减gap 且 y_bot_compare加gap大等于y_bot_val则在同一行，用空格间隔，否则就用回车换行
                if y_top_compare == 0:
                    text += result[idx][1]
                elif (
                    y_top_val - (y_top_compare - gap) >= 0
                    and (y_bot_compare + gap) - y_bot_val >= 0
                ):
                    text += " " + result[idx][1]
                else:
                    text += "\n" + result[idx][1]

                if (
                    abs(y_bot_compare - y_top_compare) <= abs(y_bot_val - y_top_val)
                    or y_top_compare == 0
                ):
                    y_top_compare, y_bot_compare = y_top_val, y_bot_val
            if args.is_translate:
                for tl in translator(text):
                    translated_text += tl["translation_text"]
                print("sequence is translated")
                pyperclip.copy(text + "\n" + translated_text)
            else:
                pyperclip.copy(text)
        else:
            print("invalid bbox")


if __name__ == "__main__":
    main()



"""
@File    :   numpy_op.py
@Time    :   2024/07/02 22:33:29
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import List, Tuple, Dict, Union
import os
import numpy as np

from rlmc.fileop.abstract_file import AbstractFile

__all__ = ["Numpy"]


class Numpy(AbstractFile):
    def __init__(self, file_path: str = "") -> None:
        self.file_path = file_path
        if file_path != "":
            self.data = self.read(file_path)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type != None:
            print(exc_type, exc_val, exc_tb)

    def read(self, file_path: str = "") -> np.ndarray:
        data = np.load(file_path)
        return data

    def write(
        self, data: np.ndarray, file_path: str, allow_pickle: bool = True
    ) -> None:
        np.save(file_path, data, allow_pickle=allow_pickle)


if __name__ == "__main__":

    with Numpy(r"./n.npy") as yl:
        print(yl.data)

    # yamlobj.write(nest_dict, "rlmc/configs/cfg1.yaml")



"""
@File    :   loss.py
@Time    :   2024/06/28 11:33:05
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import torch.nn as nn

__all__ = ["loss_method"]


loss_method = {"BCELoss": nn.BCELoss()}



"""
@File    :   handcontrol.py
@Time    :   2024/07/10 17:04:26
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import argparse
import math
import time
import copy
from typing import Union, Optional, Dict, List, Tuple, Sequence

import cv2
import pynput
from pynput.mouse import Button
from pynput.keyboard import Key
import numpy as np
import mediapipe as md
from PIL import ImageGrab


parser = argparse.ArgumentParser("handcontrol")
parser.add_argument(
    "-s",
    "--is_show",
    action="store_true",
    help="show or not",
)
parser.add_argument(
    "-f",
    "--is_show_face",
    action="store_true",
)
parser.add_argument(
    "-p",
    "--is_show_pose",
    action="store_true",
)
parser.add_argument(
    "-d",
    "--delay",
    type=float,
    default=0.2,
    help="delay seconds",
)

args = parser.parse_args()


class Command:
    def __init__(self) -> None:
        self.mouse = pynput.mouse.Controller()
        self.keyboard = pynput.keyboard.Controller()

    def _distance(self, point1: Sequence, point2: Sequence) -> float:
        return (point1[0] - point2[0]) ** 2 + (point1[1] - point2[1]) ** 2

    def condition(
        self, collector: Dict, img: np.ndarray, screen_width: int, screen_height: int
    ) -> np.ndarray:
        """judge the condition to do which command

        Args:
            collector (Dict):dict collected mediapipe results, including hand landmarks
            hand_collector = {
            0: ["WRIST", x,y]
            4: ["THUMB_TIP",x,y]
            ...
            }

            img (np.ndarray): image captured by camera

        Returns:
            img (np.ndarray)
        """
        if len(collector) >= 2:  # two hands catched by the camera
            ...

        else:  # only one hand catched by the camera
            key = list(collector.keys())[0]
            collector = collector[key]
            wrist = collector[0][1:]
            index_finger_mcp = collector[5][1:]
            thumb_tip = collector[4][1:]
            index_finger_tip = collector[8][1:]
            middle_finger_tip = collector[12][1:]
            middle_finger_pip = collector[10][1:]
            ring_finger_tip = collector[16][1:]
            # pinky_tip= collector[20][1:]

            base = self._distance(wrist, index_finger_mcp) * 0.75  # 基准距离
            if (
                self._distance(thumb_tip, middle_finger_pip) * 1.5 < base
                and self._distance(index_finger_tip, index_finger_mcp) < base
                and self._distance(middle_finger_tip, index_finger_mcp) * 0.85 < base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.7 < base
            ):
                # 握拳 移动
                self._move(wrist)

            elif (
                self._distance(thumb_tip, middle_finger_pip) * 1.5 >= base
                and self._distance(index_finger_tip, index_finger_mcp) < base
                and self._distance(middle_finger_tip, index_finger_mcp) * 0.85 < base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.7 < base
            ):
                # 竖拇指 滚动 手腕在屏幕下方1/4处往下滾动，否则往上滚动
                self._scroll(wrist, screen_height)

            elif (
                self._distance(thumb_tip, index_finger_mcp) * 1.5 > base
                and self._distance(thumb_tip, index_finger_tip) * 1.5 < base
                and self._distance(index_finger_tip, index_finger_mcp) * 1.8 >= base
                and self._distance(middle_finger_tip, index_finger_mcp) * 1.3 >= base
                and self._distance(ring_finger_tip, index_finger_mcp) * 1.1 >= base
            ):
                # 五指头合抓 压下按键（用于拖拽，要用移动和释放配合）  不灵敏待调整
                self._drag(wrist)

            elif (
                self._distance(thumb_tip, index_finger_mcp) * 1.5 > base
                and self._distance(thumb_tip, index_finger_tip) * 1.2 > base
                and self._distance(index_finger_tip, index_finger_mcp) * 1.5 >= base
                and self._distance(middle_finger_tip, index_finger_mcp) * 1.1 >= base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.9 >= base
            ):
                # 五指头分开 释放 不灵敏待调整
                self._release()

            elif (
                self._distance(thumb_tip, middle_finger_pip) * 1.3 < base
                and self._distance(index_finger_tip, index_finger_mcp) * 1.3 >= base
                and self._distance(middle_finger_tip, index_finger_mcp) * 0.85 < base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.7 < base
            ):
                # 竖食指 单击
                self._lclick()

            elif (
                self._distance(thumb_tip, index_finger_mcp) * 1.5 > base
                and self._distance(thumb_tip, index_finger_tip) * 1.2 > base
                and self._distance(index_finger_tip, index_finger_mcp) * 1.5 > base
                and self._distance(middle_finger_tip, index_finger_mcp) * 0.85 < base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.7 < base
            ):
                # 竖拇指，食指 双击
                self._double_click()

            elif (
                self._distance(thumb_tip, middle_finger_pip) * 1.2 < base
                and self._distance(index_finger_tip, index_finger_mcp) * 1.5 >= base
                and self._distance(middle_finger_tip, index_finger_mcp) * 1.2 >= base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.7 < base
            ):
                # 竖食指中指 右击
                self._rclick()

            elif (
                self._distance(thumb_tip, index_finger_mcp) * 1.3 > base
                and self._distance(thumb_tip, index_finger_tip) > base
                and self._distance(index_finger_tip, index_finger_mcp) * 1.5 >= base
                and self._distance(middle_finger_tip, index_finger_mcp) * 0.9 > base
                and self._distance(ring_finger_tip, index_finger_mcp) * 0.75 < base
            ):
                # 竖拇指，食指，中指 按alt + tab
                self._switch()

            else:
                pass

        return img

    def _move(self, point: Sequence) -> None:
        self.mouse.position = (point[0], point[1])
        # time.sleep(1)

    def _lclick(self) -> None:
        self.mouse.click(Button.left)
        time.sleep(1)

    def _rclick(self) -> None:
        self.mouse.click(Button.right)
        time.sleep(1)

    def _double_click(self) -> None:
        self.mouse.click(Button.left, 2)
        time.sleep(1)

    def _drag(self, point: Sequence) -> None:
        self.mouse.press(Button.left)
        time.sleep(1)

    def _release(self) -> None:
        self.mouse.release(Button.left)
        time.sleep(1)

    def _scroll(self, point: Sequence, screen_height: int) -> None:
        if point[1] > screen_height * 0.75:
            self.mouse.scroll(0, -1)
        else:
            self.mouse.scroll(0, 1)

    def _switch(self) -> None:
        self.keyboard.press(Key.alt)
        self.keyboard.press(Key.tab)
        self.keyboard.release(Key.alt)
        self.keyboard.release(Key.tab)
        time.sleep(1)

    def _sound_control(self) -> None:
        pass


class HandControl(Command):
    def __init__(self, camera_id: int = 0) -> None:
        super().__init__()
        self.cap = cv2.VideoCapture(camera_id)
        self.cap_width = int(self.cap.get(3))
        self.cap_height = int(self.cap.get(4))
        # self.cap_fps = self.cap.get(7)
        self.hand_detector = md.solutions.hands.Hands(max_num_hands=2)
        self.md_hands = md.solutions.hands

        self.face_mesh = md.solutions.face_mesh.FaceMesh(refine_landmarks=True)
        self.pose = md.solutions.pose.Pose()

        self.zone_size = 0.6
        self.bias = 0.1
        self.width_bias = int(self.cap_width * self.bias)
        self.height_bias = int(self.cap_height * self.bias)
        action_zone_width = int(self.cap_width * self.zone_size)
        action_zone_height = int(self.cap_height * self.zone_size)
        self.action_zone = [
            self.cap_width - action_zone_width - self.width_bias,
            self.cap_height - action_zone_height - self.height_bias,
            self.cap_width - self.width_bias,
            self.cap_height - self.height_bias,
        ]

        self.screen_width, self.screen_height = self._get_screen_size()
        self.area_radius = int(min(self.screen_width, self.screen_height) * 0.1)
        self.scale = (
            self.screen_width / action_zone_width,
            self.screen_height / action_zone_height,
        )

        self.hand_collector = {
            0: "WRIST",
            4: "THUMB_TIP",
            5: "INDEX_FINGER_MCP",
            8: "INDEX_FINGER_TIP",
            10: "MIDDLE_FINGER_PIP",
            12: "MIDDLE_FINGER_TIP",
            16: "RING_FINGER_TIP",
            20: "PINKY_TIP",
        }

        # self.hand_dict = {
        #     "WRIST": 0,
        #     "THUMB_CMC": 1,
        #     "THUMB_MCP": 2,
        #     "THUMB_IP": 3,
        #     "THUMB_TIP": 4,
        #     "INDEX_FINGER_MCP": 5,
        #     "INDEX_FINGER_PIP": 6,
        #     "INDEX_FINGER_DIP": 7,
        #     "INDEX_FINGER_TIP": 8,
        #     "MIDDLE_FINGER_MCP": 9,
        #     "MIDDLE_FINGER_PIP": 10,
        #     "MIDDLE_FINGER_DIP": 11,
        #     "MIDDLE_FINGER_TIP": 12,
        #     "RING_FINGER_MCP": 13,
        #     "RING_FINGER_PIP": 14,
        #     "RING_FINGER_DIP": 15,
        #     "RING_FINGER_TIP": 16,
        #     "PINKY_MCP": 17,
        #     "PINKY_PIP": 18,
        #     "PINKY_DIP": 19,
        #     "PINKY_TIP": 20,
        # }

    def _get_screen_size(self) -> Tuple[int, int]:
        screen = ImageGrab.grab()
        screen_width, screen_height = screen.size
        return int(screen_width), int(screen_height)

    def _hand(self, img: np.ndarray) -> np.ndarray:
        self.hand_data = self.hand_detector.process(img)
        cv2.rectangle(
            img,
            self.action_zone[:2],
            self.action_zone[2:],
            (170, 234, 242),
            5,
            lineType=cv2.LINE_AA,
        )
        if self.hand_data.multi_hand_landmarks:
            hands_collector = dict()
            for i, handlms in enumerate(self.hand_data.multi_hand_landmarks):
                if args.is_show:
                    md.solutions.drawing_utils.draw_landmarks(
                        img, handlms, self.md_hands.HAND_CONNECTIONS
                    )

                if i == 1:
                    left_hand_collector = dict()
                    # hand_dict = dict()
                    for idx, lm in enumerate(handlms.landmark):
                        # hand_dict[md.solutions.hands.HandLandmark(idx).name] = idx
                        # print(hand_dict)
                        if idx in self.hand_collector.keys():
                            x, y = int(lm.x * self.cap_width), int(
                                lm.y * self.cap_height
                            )
                            # cv2.putText(img, 'left', (x, y), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 0, 255), 1, cv2.LINE_AA)
                            x, y = self._mapping(x, y)
                            left_hand_collector[idx] = [self.hand_collector[idx], x, y]

                    if len(self.hand_collector) == len(left_hand_collector):
                        hands_collector["left"] = left_hand_collector
                elif i == 0:
                    right_hand_collector = dict()
                    # hand_dict = dict()
                    for idx, lm in enumerate(handlms.landmark):
                        # hand_dict[md.solutions.hands.HandLandmark(idx).name] = idx
                        # print(hand_dict)
                        if idx in self.hand_collector.keys():
                            x, y = int(lm.x * self.cap_width), int(
                                lm.y * self.cap_height
                            )
                            # cv2.putText(img, 'right', (x, y), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 0, 255), 1, cv2.LINE_AA)
                            x, y = self._mapping(x, y)
                            # print(x,y)
                            right_hand_collector[idx] = [self.hand_collector[idx], x, y]
                    if len(self.hand_collector) == len(right_hand_collector):
                        hands_collector["right"] = right_hand_collector
                else:
                    print("too many hands")

            # 因后续以wrist作为移动点，将wrist超屏幕范围的剔除
            if "left" in hands_collector.keys():
                point = hands_collector["left"][0][1:]
                if not self._check_screen_xy(point[0], point[1]):
                    del hands_collector["left"]

            if "right" in hands_collector.keys():
                point = hands_collector["right"][0][1:]
                if not self._check_screen_xy(point[0], point[1]):
                    del hands_collector["right"]
            # command
            if len(hands_collector) >= 1:
                img = self.condition(
                    hands_collector, img, self.screen_width, self.screen_height
                )

        return img

    def _face(self, img: np.ndarray) -> np.ndarray:
        results = self.face_mesh.process(img)
        if results.multi_face_landmarks:
            for face_landmarks in results.multi_face_landmarks:
                # 脸网格
                md.solutions.drawing_utils.draw_landmarks(
                    image=img,
                    landmark_list=face_landmarks,
                    connections=md.solutions.face_mesh.FACEMESH_TESSELATION,
                    landmark_drawing_spec=None,
                    connection_drawing_spec=md.solutions.drawing_styles.get_default_face_mesh_tesselation_style(),
                )
                # 人脸
                md.solutions.drawing_utils.draw_landmarks(
                    image=img,
                    landmark_list=face_landmarks,
                    connections=md.solutions.face_mesh.FACEMESH_CONTOURS,
                    landmark_drawing_spec=None,
                    connection_drawing_spec=md.solutions.drawing_styles.get_default_face_mesh_contours_style(),
                )
                # 瞳孔
                md.solutions.drawing_utils.draw_landmarks(
                    image=img,
                    landmark_list=face_landmarks,
                    connections=md.solutions.face_mesh.FACEMESH_IRISES,
                    landmark_drawing_spec=None,
                    connection_drawing_spec=md.solutions.drawing_styles.get_default_face_mesh_iris_connections_style(),
                )

        return img

    def _pose(self, img: np.ndarray) -> np.ndarray:
        results = self.pose.process(img)
        if results.pose_landmarks:
            for pose_landmarks in results.pose_landmarks.landmark:
                md.solutions.drawing_utils.draw_landmarks(
                    image=img,
                    landmark_list=results.pose_landmarks,
                    connections=md.solutions.pose.POSE_CONNECTIONS,
                )

        return img

    def _mapping(self, x: Union[int, float], y: Union[int, float]) -> Tuple[int, int]:
        return int((x - self.action_zone[0]) * self.scale[0]), int(
            (y - self.action_zone[1]) * self.scale[1]
        )

    def _check_is_in_area(
        self,
        point: Sequence,
        area_center_point: Sequence,
        area_radius: Union[float, int],
    ) -> bool:
        distance = (point[0] - area_center_point[0]) ** 2 + (
            point[1] - area_center_point[1]
        ) ** 2
        return distance <= area_radius**2

    def _check_screen_xy(self, x: Union[int, float], y: Union[int, float]):
        if 0 <= x <= self.screen_width and 0 <= y <= self.screen_height:
            return True
        else:
            return False

    def run(self) -> None:
        while True:
            success, img = self.cap.read()
            img = cv2.flip(img, 1)
            img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            img_rgb = self._hand(img_rgb)

            if args.is_show_face:
                img_rgb = self._face(img_rgb)

            if args.is_show_pose:
                img_rgb = self._pose(img_rgb)

            if args.is_show:
                img = cv2.cvtColor(img_rgb, cv2.COLOR_RGB2BGR)
                cv2.imshow("Camera", img)
                if cv2.waitKey(1) & 0xFF == 27:
                    break

            time.sleep(args.delay)

        self.cap.release()
        cv2.destroyAllWindows()


def main() -> None:
    handcontrol = HandControl()
    handcontrol.run()


if __name__ == "__main__":
    main()



"""
@File    :   video_op.py
@Time    :   2024/07/03 08:59:38
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

import av.stream

if "." not in sys.path:
    sys.path.append(".")

import os
from typing import Literal

import av
import cv2
import numpy as np

from rlmc.fileop.abstract_file import AbstractFile

__all__ = ["VideoFile"]


# 人脸检测器
class FaceDetector:
    def __init__(self, module_file):
        self.module_file = module_file
        self.face_cascade = cv2.CascadeClassifier(self.module_file)

    def detectFace(self, gray_img):
        face_rect = self.face_cascade.detectMultiScale(gray_img, 1.3, 5)
        return face_rect


class VideoFile:
    def __init__(
        self,
        file_path: str = "",
        # mode: Literal["stream", "camera", "file"] = "file",
        vcodec: Literal["mpeg4", "libx264"] = "libx264",
        pixformat: Literal["rgb24"] = "rgb24",
        width: int = 0,
        height: int = 0,
        fps: int = 30,
        duration: float = 0.0,
        acodec: Literal["aac", "mp3"] = "aac",  #
        bit_rate: int = 2_000_000,
        audio_channels: int = 1,
        audio_sample_rate: int = 44100,
        audio_sample_fmt: str = "",
        is_save: bool = False,
        save_path: str = "",
        tread_type: Literal["SLICE", "AUTO"] = "SLICE",
    ) -> None:

        self.container = None
        self.output_container = None
        self.file_path = file_path
        self.vcodec = vcodec
        self.pixformat = pixformat
        self.acodec = acodec
        self.bit_rate = bit_rate
        self.audio_channels = audio_channels
        self.audio_sample_rate = audio_sample_rate
        self.audio_sample_fmt = audio_sample_fmt
        self.audio_format_dtypes = {
            "dbl": "<f8",
            "dblp": "<f8",
            "flt": "<f4",
            "fltp": "<f4",
            "s16": "<i2",
            "s16p": "<i2",
            "s32": "<i4",
            "s32p": "<i4",
            "u8": "u1",
            "u8p": "u1",
        }
        self.is_save = is_save
        self.save_path = save_path
        self.fps = fps
        self.duration = duration
        self.total_frames = 0
        self.tread_type = tread_type
        self.width = width
        self.height = height

        if file_path != "":
            self.container = self.read()

    def __enter__(self):
        self.read()
        if self.is_save:
            self.output_container = av.open(self.save_path, "w")
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.container.close()
        if self.is_save:
            self.output_container.close()
        if exc_type != None:
            print(exc_type, exc_val, exc_tb)

    def add_vid_stream(self):
        output_stream = self.output_container.add_stream(self.vcodec, rate=self.fps)
        codec = output_stream.codec_context
        codec.width = self.width
        codec.height = self.height
        codec.pix_fmt = self.pixformat
        codec.bit_rate = self.bit_rate
        return output_stream

    def add_aud_stream(self):
        output_stream = self.output_container.add_stream(
            self.acodec, rate=self.audio_sample_rate
        )
        return output_stream

    def read(self):
        cap = cv2.VideoCapture(self.file_path)
        self.total_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
        self.fps = cap.get(cv2.CAP_PROP_FPS)
        self.duration = self.total_frames / self.fps
        cap.release()
        self.container = av.open(self.file_path)
        self.bit_rate = self.container.streams.video[0].codec_context.bit_rate
        self.pixformat = self.container.streams.video[0].codec_context.pix_fmt
        self.width = self.container.streams.video[0].codec_context.width
        self.height = self.container.streams.video[0].codec_context.height
        self.audio_sample_rate = self.container.streams.audio[
            0
        ].codec_context.sample_rate
        self.audio_channels = self.container.streams.audio[0].codec_context.channels
        self.audio_sample_fmt = self.container.streams.audio[0].format.name
        return self.container


if __name__ == "__main__":
    module_file = r"D:\RLdownload\haarcascade_frontalface_alt2.xml"
    src_video = r"C:\Users\\Desktop\ai_design\cctv.mp4"
    is_show = True
    is_save = True
    save_path = r"C:\Users\\Desktop\cctv_new.mp4"

    dectector = FaceDetector(module_file)

    with VideoFile(
        src_video,
        is_save=is_save,
        save_path=save_path,
    ) as f:

        # container = f.read()
        out_vid_stream = f.add_vid_stream()
        out_aud_stream = f.add_aud_stream()
        flag = False
        for packet in f.container.demux():
            if packet.stream.type == "audio":
                for frame in packet.decode():
                    format_dtype = np.dtype(f.audio_format_dtypes[f.audio_sample_fmt])
                    layout = "stereo" if f.audio_channels > 1 else "mono"
                    print("index:", frame.index)
                    print("时间戳:", frame.pts)
                    print("channels:", f.audio_channels)
                    print("采样率:", frame.sample_rate)
                    print("采样格式:", f.audio_sample_fmt)
                    print(frame.to_ndarray().shape)
                    new_frame = av.AudioFrame.from_ndarray(
                        frame.to_ndarray(), format=f.audio_sample_fmt, layout=layout
                    )
                    new_frame.sample_rate = f.audio_sample_rate
                    for packet in out_aud_stream.encode(new_frame):
                        f.output_container.mux(packet)

            elif packet.stream.type == "video":
                for frame in packet.decode():
                    print("index:", frame.index)
                    print("时间戳:", frame.pts)
                    print("帧类型:", frame.pict_type)
                    print("宽度:", frame.width)
                    print("高度:", frame.height)
                    img_ndarray = frame.to_rgb().to_ndarray()
                    print("像素数据:", img_ndarray.shape)

                    # 检测人脸
                    gray = cv2.cvtColor(img_ndarray, cv2.COLOR_RGB2GRAY)
                    faces = dectector.detectFace(gray)
                    for x, y, w, h in faces:
                        cv2.rectangle(
                            img_ndarray, (x, y), (x + w, y + h), (0, 255, 0), 2
                        )

                    new_frame = av.VideoFrame.from_ndarray(img_ndarray, format="rgb24")
                    for packet in out_vid_stream.encode(new_frame):
                        f.output_container.mux(packet)

                    if is_show:
                        img_ndarray = cv2.cvtColor(img_ndarray, cv2.COLOR_RGB2BGR)
                        cv2.imshow("frame", img_ndarray)
                        if cv2.waitKey(1) & 0xFF == ord("q"):
                            flag = True
                            break
            if flag:
                break
        # flush stream
        for packet in out_vid_stream.encode():
            f.output_container.mux(packet)
        for packet in out_aud_stream.encode():
            f.output_container.mux(packet)
        if is_show:
            cv2.destroyAllWindows()



import torch
from torchvision import transforms
from timm import create_model, list_models
from PIL import Image


model_names = list_models(pretrained=True)
# print(f'available models:\n {model_names}')


# # 选择一个预训练模型
# model_name = 'deeplabv3_resnet50'
# pretrained_model = create_model(model_name, pretrained=True)

# # 切换到评估模式，关闭dropout和batch normalization层
# pretrained_model.eval()

# # 定义预处理变换
# transform = transforms.Compose([
#     transforms.Resize(224),
#     transforms.ToTensor(),
#     transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]),
# ])

# # 加载图像
# image_path = r"C:\Users\\Desktop\ai_design\00019-2600014894.png"
# image = Image.open(image_path).convert('RGB')

# # 应用预处理变换
# image_tensor = transform(image).unsqueeze(0)  # 添加batch维度

# # 如果有GPU，将图像和数据模型转移到GPU
# device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
# image_tensor = image_tensor.to(device)
# pretrained_model = pretrained_model.to(device)

# # 提取特征
# with torch.no_grad():  # 不需要计算梯度，节省内存和计算资源
#     features = pretrained_model.forward_features(image_tensor)  # 获取特征

# # 将特征转移到CPU（如果需要）并展平
# features = features.cpu().numpy().flatten()

# print(features)
# print(features.max())
# print(features.min())

#####################################################################################
model = create_model("resnet50", pretrained=True)

# 将模型设置为评估模式
model.eval()

# 加载图像并进行预处理
image = Image.open(r"C:\Users\\Desktop\ai_design\z2.jpg")
# 定义图像预处理转换
preprocess = transforms.Compose(
    [
        transforms.Resize((256, 256)),  # 调整图像大小为256x256
        transforms.ToTensor(),  # 转换为Tensor类型
        transforms.Normalize(
            mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]
        ),  # 归一化
    ]
)

# 应用预处理转换
processed_image = preprocess(image)

# 查看处理后的图像形状和数值范围
print("Processed image shape:", processed_image.shape)
print("Processed image range:", processed_image.min(), "-", processed_image.max())

# 5. 可选：将处理后的图像转换回PIL图像对象
processed_pil_image = transforms.ToPILImage()(processed_image)

# 6. 可选：显示处理后的图像
processed_pil_image.show()


# 将输入张量转换为批处理张量
input_batch = processed_image.unsqueeze(0)

# 将输入张量传递给模型并获取输出
with torch.no_grad():
    output = model(input_batch)

# 获取预测结果
_, predicted = torch.max(output.data, 1)

# 打印预测结果
print(predicted)
print(predicted.item())



"""
@File    :   stt.py
@Time    :   2024/07/10 09:57:46
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Optional, Union, List, Dict
import argparse
import time

import numpy as np
import torch
import pyaudio
import pynput
import pyperclip
from faster_whisper import WhisperModel
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM, pipeline


parser = argparse.ArgumentParser("stt")
parser.add_argument(
    "-r",
    "--record_time",
    default=5,
    type=int,
    help="record time(second)",
)
parser.add_argument(
    "-l",
    "--record_src_language",
    default="zh",
    help="record src language",
)

parser.add_argument("-i", "--is_translate", action="store_true", help="is_translate")
parser.add_argument(
    "-s",
    "--translate_src_lang",
    default="zho_Hans",
    help="translate zho_Hans, zho_Hant,eng_Latn, jpn_Jpan,yue_Hant,kor_Hang",
)
parser.add_argument(
    "-t",
    "--translate_tgt_lang",
    default="eng_Latn",
    help="translate zho_Hans, zho_Hant,eng_Latn, jpn_Jpan,yue_Hant,kor_Hang",  # zho_Hant繁体
)
parser.add_argument(
    "-m",
    "--translate_model",
    default="facebook/nllb-200-distilled-600M",
    help="translate model",
)
args = parser.parse_args()

signal = 0
INT16_MAX_ABS_VALUE = 32768.0
CHUNK = 512
FORMAT = pyaudio.paInt16
CHANNELS = 1
RATE = 16000


model_size = "large-v3"
device = "cuda" if torch.cuda.is_available() else "cpu"
stt_model = WhisperModel(model_size, device=device, compute_type="float16")


def on_press(key: pynput.keyboard.Key) -> Optional[bool]:
    if key == pynput.keyboard.Key.esc:
        global signal
        signal = 1
        return False
    elif hasattr(key, "char"):
        if key.char == "f":
            return False
    # print("Key {0} pressed".format(key))


def kb_listener() -> None:
    listener = pynput.keyboard.Listener(on_press=on_press)
    listener.start()
    listener.join()


def speech_to_text() -> str:
    # 创建一个PyAudio对象
    p = pyaudio.PyAudio()

    # 打开麦克风流
    stream = p.open(
        format=FORMAT,
        channels=CHANNELS,
        rate=RATE,
        input=True,
        output=True,
        frames_per_buffer=CHUNK,
    )

    audio_collector = []
    counter = 0
    unit = RATE // CHUNK
    text = ""
    start = time.time()
    print("start record...")
    while True:
        audio_data = stream.read(CHUNK)
        audio_collector.append(audio_data)
        counter += 1
        if counter % (int(unit * args.record_time) + 1) == 0:
            audio_collector = np.frombuffer(b"".join(audio_collector), dtype=np.int16)
            audio_collector = audio_collector.astype(np.float32) / INT16_MAX_ABS_VALUE
            print("record time: ", time.time() - start)
            segments, info = stt_model.transcribe(
                audio_collector,
                language=args.record_src_language,
                beam_size=5,
                log_prob_threshold=-0.5,
            )

            text = ""
            for segment in segments:
                # print("[%.2fs -> %.2fs] %s" % (segment.start, segment.end, segment.text))
                if segment.text != "":
                    text += segment.text
            # initial
            audio_collector = []
            counter = 0
            break

    # 关闭流和PyAudio对象
    stream.stop_stream()
    stream.close()
    p.terminate()

    return text


def main() -> None:
    global signal

    print("record_lang: ", args.record_src_language)

    if args.is_translate:
        tokenizer = AutoTokenizer.from_pretrained(args.translate_model)
        translate_model = AutoModelForSeq2SeqLM.from_pretrained(args.translate_model)

        translator = pipeline(
            "translation",
            model=translate_model,
            tokenizer=tokenizer,
            src_lang=args.translate_src_lang,
            tgt_lang=args.translate_tgt_lang,
            max_length=512,
            device=0,
        )
        print(
            "translate model is ready, src_lang: ",
            args.translate_src_lang,
            "tgt_lang: ",
            args.translate_tgt_lang,
        )

    while True:
        kb_listener()
        if signal == 1:
            break
        text = speech_to_text()
        print("record is done")
        translated_text = ""
        if args.is_translate and text != "":
            for tl in translator(text):
                translated_text += tl["translation_text"]
            print("sequence is translated")
            text = translated_text
        pyperclip.copy(text)


if __name__ == "__main__":
    main()



from typing import List

import cv2
import numpy as np
import torch
from torchvision.transforms import Normalize

__all__ = [
    "invert_transform",
    "onehot2mask",
    "mask2onehot",
    "letterbox_image",
]


def invert_transform(
    torch_normalize_tensor: torch.Tensor, mean: List[float], std: List[float]
) -> torch.Tensor:
    """
    反归一化同时调整成 h,w,c
    """
    mean_ = torch.tensor(mean)
    std_ = torch.tensor(std)
    MEAN = [-mean / std for mean, std in zip(mean_, std_)]
    STD = [1 / std for std in std_]
    denormalize = Normalize(mean=MEAN, std=STD)
    tensor = denormalize(torch_normalize_tensor) * 255
    tensor = tensor.permute(1, 2, 0).long()
    return tensor


def onehot2mask(torch_tensor: torch.Tensor, axis: int = 0) -> torch.Tensor:
    mask = torch.argmax(torch_tensor, axis=axis)
    return mask


def mask2onehot(mask: np.ndarray, num_classes: int) -> np.ndarray:
    """
    (H,W) to (K,H,W)
    """
    mask_ = [mask == i for i in range(num_classes)]
    mask_ = np.array(mask_)
    return mask_


def letterbox_image(
    image: np.ndarray, resize_width: int, resize_height: int
) -> np.ndarray:
    """
    图片统一尺寸，补灰条操作
    :param: image
    :return: new_image
    """
    ih = image.shape[0]
    iw = image.shape[1]

    w = resize_width
    h = resize_height

    scale = min(w / iw, h / ih)
    nw = int(iw * scale)
    nh = int(ih * scale)

    image = cv2.resize(image, (nw, nh))
    new_image = np.full(
        (h, w, 3), 128, dtype=image.dtype
    )  # channels为3，仅jpg和三通道图

    new_image[
        (h - nh) // 2 : (h - nh) // 2 + nh, (w - nw) // 2 : (w - nw) // 2 + nw, :
    ] = image
    # print(new_image.shape)
    return new_image



"""
@File    :   mat_op.py
@Time    :   2024/07/16 17:53:32
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import List, Tuple, Dict, Union
import os
import scipy.io as scio

from rlmc.fileop.abstract_file import AbstractFile

__all__ = ["Mat"]


class Mat(AbstractFile):
    def __init__(self, file_path: str = "") -> None:
        self.file_path = file_path
        if file_path != "":
            self.data = self.read(file_path)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type != None:
            print(exc_type, exc_val, exc_tb)

    def read(self, file_path: str) -> Dict:
        data = scio.loadmat(file_path)
        print("mat_keys:", data.keys())
        return data

    def write(self, data: Dict, file_path: str) -> None:
        scio.savemat(file_path, data)



# -*- coding:utf-8 -*-
# @Time    : 2023/1/13 23:20


import torch.nn as nn
import torch.optim as optim
from datasets import *
from transformer import Transformer

if __name__ == "__main__":

    enc_inputs, dec_inputs, dec_outputs = make_data()
    loader = Data.DataLoader(MyDataSet(enc_inputs, dec_inputs, dec_outputs), 2, True)

    model = Transformer().cuda()
    criterion = nn.CrossEntropyLoss(ignore_index=0)         # 忽略 占位符 索引为0.
    optimizer = optim.SGD(model.parameters(), lr=1e-3, momentum=0.99)

    for epoch in range(50):
        for enc_inputs, dec_inputs, dec_outputs in loader:  # enc_inputs : [batch_size, src_len]
                                                            # dec_inputs : [batch_size, tgt_len]
                                                            # dec_outputs: [batch_size, tgt_len]

            enc_inputs, dec_inputs, dec_outputs = enc_inputs.cuda(), dec_inputs.cuda(), dec_outputs.cuda()
            outputs, enc_self_attns, dec_self_attns, dec_enc_attns = model(enc_inputs, dec_inputs)
                                                            # outputs: [batch_size * tgt_len, tgt_vocab_size]
            loss = criterion(outputs, dec_outputs.view(-1))
            print('Epoch:', '%04d' % (epoch + 1), 'loss =', '{:.6f}'.format(loss))
            optimizer.zero_grad()
            loss.backward()
            optimizer.step()
    torch.save(model, 'model.pth')
    print("保存模型")


"""
@File    :   json_op.py
@Time    :   2024/07/02 22:13:11
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Union, Dict, List
import os
import json

from rlmc.fileop.abstract_file import AbstractFile

__all__ = ["Json"]


class Json(AbstractFile):
    def __init__(
        self, file_path: str = "", mode: str = "r", encoding: str = "utf-8"
    ) -> None:
        self.file_path = file_path
        self.encoding = encoding
        self.mode = mode
        if file_path != "":
            self.data = self.read(file_path)

    def __enter__(self):
        self.file = open(self.file_path, self.mode, encoding=self.encoding)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.file.close()
        if exc_type != None:
            print(exc_type, exc_val, exc_tb)

    def read(self, file_path: str = "") -> Union[Dict, List]:
        with open(file_path, self.mode, encoding=self.encoding) as f:
            data = json.load(f)
        return data

    def write(
        self,
        data: Union[Dict, List],
        file_path: str,
        mode: str = "w",
        encoding: str = "utf-8",
        indent: int = 2,
    ) -> None:
        with open(file_path, mode, encoding=encoding) as f:
            json.dump(data, f, ensure_ascii=False, indent=indent)



"""
@File    :   multipro.py
@Time    :   2024/06/20 16:38:18
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if '.' not in sys.path: sys.path.append(".")

from typing import Any, Union, Callable, Iterable, List, Dict
import multiprocessing as mp
import time

from tqdm import tqdm
from rlmc.utils.logger import Logger


__all__ = ["MultiProcess"]

logger = Logger(__name__, level=Logger.DEBUG)


class MultiProcess:
    """
    多进程建议用py运行，在jupyter可能会报错或无返回数据,要生成py再运行,另外win多进程要有if __name__=='__main__':
    """

    def __init__(self, func: Callable, worker: int) -> None:
        self.func = func
        self.worker = worker

    def manager(self) -> None:
        mgr = mp.Manager()
        self.mgrd = mgr.dict()

    def unpack_args(self, args: Union[tuple, dict]) -> Callable:
        if isinstance(args, tuple):
            return self.func(*args)
        elif isinstance(args, dict):
            return self.func(**args)
        else:
            raise TypeError("args must be tuple or dict")

    def run(self, tasklist: Iterable[Any], chunksize: int) -> List:
        pool = mp.Pool(processes=self.worker)
        try:
            res = list(tqdm(pool.imap(self.unpack_args, tasklist, chunksize=chunksize)))
            pool.close()  # 执行join()前必须执行close(),表示不能继续添加新的进程了
            pool.join()
        except:
            pool.terminate()
            res = []
        finally:
            return res


def test_unit(a, b):
    return a + b, b


def test_unit_no_return(a, b):
    print(a + b, b)


if __name__ == "__main__":

    a = [i for i in range(100)]
    b = [i + 1 for i in range(100)]
    task_list = list(zip(a, b))

    task_list = []
    for i in range(1000):
        task_list.append({"a": i, "b": i + 1})
    # print(task_list)

    mup = MultiProcess(test_unit, 4)
    res = mup.run(task_list, 20)
    print(res)




from typing import List
import time
import argparse

import ChatTTS
import torch
import torchaudio
import pyaudio
import numpy as np
from scipy.signal import resample


parser = argparse.ArgumentParser("tts")
parser.add_argument(
    "-t",
    "--texts",
    nargs="+",
    required=True,
    help="List of texts"
)

parser.add_argument(
    "--seed",
    default=17,  #19
    type=int,
    help="speaker seed"
)

parser.add_argument(
    "-s",
    "--is_save",
    action="store_true",
    default=False,
)

args = parser.parse_args()

SEED = args.seed
IS_SAVE = args.is_save
IS_DISPLAY = True

CHANNELS = 1
RATE = 16000
FORMAT = pyaudio.paInt16
CHUNK = 6000

resample_factor = RATE / 24000  # chattts采样率是24000，重采样为RATE



def main() -> None:
    texts = args.texts
    print(texts)

    chat = ChatTTS.Chat()
    chat.load(compile=False)  # Set to True for better performance
    torch.manual_seed(SEED)
    rand_spk = chat.sample_random_speaker()

    params_infer_code = ChatTTS.Chat.InferCodeParams(
        spk_emb=rand_spk,  # add sampled speaker
        temperature=0.3,
        top_P=0.7,
        top_K=20,
    )

    params_refine_text = ChatTTS.Chat.RefineTextParams(
        prompt="[oral_2][laugh_0][break_4]",
    )

    wavs = chat.infer(
        texts,
        # skip_refine_text=True,
        params_refine_text=params_refine_text,
        params_infer_code=params_infer_code,
    )

    print(wavs[0].shape)

    if IS_DISPLAY:
        p = pyaudio.PyAudio()
        # 打开麦克风流
        stream = p.open(
            format=FORMAT,
            channels=CHANNELS,
            rate=RATE,
            input=True,
            output=True,
            frames_per_buffer=CHUNK,
        )

        for i in range(len(wavs)):
            signal = wavs[i].squeeze()
            resampled_signal = resample(signal, int(resample_factor * signal.shape[0]))  # 重采样
            output = (resampled_signal * 32768).astype(np.int16)[:, np.newaxis]
            stream.write(output.tobytes())  # np.ndarray to bytes，否则声音卡顿

        stream.stop_stream()
        stream.close()

        p.terminate()

    if IS_SAVE:
        for i in range(len(wavs)):
            """
            In some versions of torchaudio, the first line works but in other versions, so does the second line.
            """
            try:
                torchaudio.save(
                    f"basic_output{i}.wav", torch.from_numpy(wavs[i]).unsqueeze(0), RATE
                )
            except:
                torchaudio.save(f"basic_output{i}.wav", torch.from_numpy(wavs[i]), RATE)

if __name__ == "__main__":
    main()


"""
@File    :   live.py
@Time    :   2024/07/04 00:00:14
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import multiprocessing as mp
from os import remove, mkdir, listdir
from os.path import exists, splitext, basename, join
from datetime import datetime
import time
from fractions import Fraction

import pyaudio
from PIL import ImageGrab
import cv2
import av
import numpy as np


MODE = "screen"  # 'screen' or 'camera'
IS_DISPLAY = False
IS_SAVE = True
CHUNK_SIZE = 4096
CHANNELS = 1
FORMAT = pyaudio.paFloat32  # pyaudio.paInt16
RATE = 48000


def record_audio(mgr_dict, mgr_audio_data):
    p = pyaudio.PyAudio()
    time.sleep(3)
    start = time.time()
    stream = p.open(
        format=FORMAT,
        channels=CHANNELS,
        rate=RATE,
        input=True,
        # input_device_index=4,#立体混音或其他
        output=True,
        frames_per_buffer=CHUNK_SIZE,
    )

    while mgr_dict["allowRecording"]:
        data = stream.read(CHUNK_SIZE)
        if IS_DISPLAY:
            stream.write(data)
        if IS_SAVE:
            np_data = np.frombuffer(data, dtype=np.float32)
            pts = int((time.time() - start) * RATE)
            mgr_audio_data[pts] = np_data

    stream.stop_stream()
    stream.close()
    p.terminate()


def record_screen(mgr_dict, mgr_audio_data, output_path):
    # 录制屏幕
    time.sleep(3)
    start = time.time()
    im = ImageGrab.grab()
    im = np.array(im)
    fps = 30
    h, w, c = im.shape

    if IS_SAVE:
        container = av.open(output_path, mode="w", format="mp4")
        stream = container.add_stream("h264", rate=fps) 
        stream.width = int(w)
        stream.height = int(h)
        stream.pix_fmt = "yuv420p" # 'nv16'   #
        stream.bit_rate=2_000_000
        stream.codec_context.time_base = Fraction(1, int(fps))

        audio_stream = container.add_stream("mp3", rate=RATE)
        audio_stream.codec_context.time_base = Fraction(1, int(RATE))
        layout = "stereo" if CHANNELS > 1 else "mono"

    while mgr_dict["allowRecording"]:
        im = ImageGrab.grab()
        im = np.array(im)
        if IS_DISPLAY:
            cv2.imshow("im", cv2.cvtColor(im, cv2.COLOR_RGB2BGR))
            cv2.waitKey(1)
        if IS_SAVE:
            im_frame = av.VideoFrame.from_ndarray(im, format="rgb24")
            for packet in stream.encode(im_frame):
                container.mux(packet)

            np_data_dict = mgr_audio_data

            for pts, np_data in np_data_dict.items():
                if np_data.ndim == 1:
                    np_data = np.reshape(np_data, (1, -1))
                new_frame = av.AudioFrame.from_ndarray(
                    np_data, format="fltp", layout=layout
                )

                new_frame.sample_rate = RATE
                new_frame.pts = pts

                for packet in audio_stream.encode(new_frame):
                    container.mux(packet)

                mgr_audio_data.pop(pts)

    if IS_DISPLAY:
        cv2.destroyAllWindows()
    if IS_SAVE:
        # Flush stream
        for packet in stream.encode():
            container.mux(packet)
        for packet in audio_stream.encode():
            container.mux(packet)

        # Close the file
        container.close()


def record_cam(mgr_dict, mgr_audio_data, output_path):
    time.sleep(3)
    start = time.time()

    cap = cv2.VideoCapture(0)  # camera id
    frame_height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    frame_width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    fps = cap.get(cv2.CAP_PROP_FPS)

    if IS_SAVE:
        container = av.open(output_path, mode="w",format="mp4")
        stream = container.add_stream("h264", rate=fps)
        stream.width = frame_width
        stream.height = frame_height
        stream.pix_fmt = "yuv420p"
        stream.bit_rate=2_000_000
        stream.codec_context.time_base = Fraction(1, int(fps))

        audio_stream = container.add_stream("mp3", rate=RATE)
        audio_stream.codec_context.time_base = Fraction(1, int(RATE))
        layout = "stereo" if CHANNELS > 1 else "mono"

    while cap.isOpened() and mgr_dict["allowRecording"]:
        ret, im = cap.read()
        if ret:
            if IS_DISPLAY:
                cv2.imshow("im", im)
                cv2.waitKey(1)
            if IS_SAVE:
                im_frame = av.VideoFrame.from_ndarray(
                    cv2.cvtColor(im, cv2.COLOR_BGR2RGB), format="rgb24"
                )
                for packet in stream.encode(im_frame):
                    container.mux(packet)

                np_data_dict = mgr_audio_data

                for pts, np_data in np_data_dict.items():
                    if np_data.ndim == 1:
                        np_data = np.reshape(np_data, (1, -1))
                    new_frame = av.AudioFrame.from_ndarray(
                        np_data, format="fltp", layout=layout
                    )

                    new_frame.sample_rate = RATE
                    new_frame.pts = pts

                    for packet in audio_stream.encode(new_frame):
                        container.mux(packet)

                    mgr_audio_data.pop(pts)
        else:
            break

    cap.release()
    if IS_DISPLAY:
        cv2.destroyAllWindows()
    if IS_SAVE:
        # Flush stream
        for packet in stream.encode():
            container.mux(packet)
        for packet in audio_stream.encode():
            container.mux(packet)

        # Close the file
        container.close()


root = r"C:\Users\\Desktop\temp_file"
now = str(datetime.now())[:19].replace(":", "_")
video_output = join(root, "%s.mp4" % now)


if __name__ == "__main__":
    mgr_dict = mp.Manager().dict()
    mgr_dict["allowRecording"] = True
    mgr_audio_data = mp.Manager().dict()

    t1 = mp.Process(target=record_audio, args=(mgr_dict, mgr_audio_data))
    if MODE == "screen":
        t2 = mp.Process(
            target=record_screen, args=(mgr_dict, mgr_audio_data, video_output)
        )
    elif MODE == "camera":
        t2 = mp.Process(
            target=record_cam, args=(mgr_dict, mgr_audio_data, video_output)
        )

    for t in (t1, t2):
        t.daemon = True

    for t in (t1, t2):
        t.start()

    print("3秒后开始录制，按q键回车结束录制")
    while True:
        if input() == "q":
            break

    mgr_dict["allowRecording"] = False
    for i in (t1, t2):
        t.join()

    print("End")



import sys

if "." not in sys.path:
    sys.path.append(".")

import argparse
import os

from rlmc.utils.downloadscript import HfDownload, MsDownload, AutodlDownload


parser = argparse.ArgumentParser("download model")
parser.add_argument("-m", "--method", default="hf", help="hf,ms,autodl")
parser.add_argument("-r", "--repo_id", required=True, help="repo id")
parser.add_argument("-x", "--XDG_CACHE_HOME", default="", help="XDG_CACHE_HOME")
parser.add_argument("-c", "--cache_dir", default="", help="cache dir")
parser.add_argument("-l", "--is_login", default=False, help="is login hf")
parser.add_argument("-t", "--hf_token", default="", help="hf token")
parser.add_argument("-i", "--ignore_patterns", default=[], nargs="+", help="*.h5  *safetensors  *msgpack")

args = parser.parse_args()

if args.XDG_CACHE_HOME:
    os.environ["XDG_CACHE_HOME"] = args.XDG_CACHE_HOME
    os.environ["MODELSCOPE_CACHE"] = os.path.join(args.XDG_CACHE_HOME, "modelscope")


def main() -> None:
    print("sys XDG_CACHE_HOME: ", os.environ.get("XDG_CACHE_HOME"))
    if args.method == "hf":
        dl = HfDownload(
            args.repo_id, args.cache_dir, ignore_patterns=args.ignore_patterns, hf_token=args.hf_token, is_login=args.is_login
        )
    elif args.method == "ms":
        dl = MsDownload(args.repo_id, args.cache_dir,ignore_file_pattern=args.ignore_patterns)
    else:
        dl = AutodlDownload(args.repo_id, args.cache_dir)
    dl.run()



import argparse

from moviepy.editor import VideoFileClip


parser = argparse.ArgumentParser("cut video")

parser.add_argument(
    "-i", "--input_file", default="", required=True, help="video file", type=str
)
parser.add_argument(
    "-o", "--output_file", default="./output.mp4", type=str, help="output file"
)
parser.add_argument("-s", "--start", default=0.0, type=float, help="start second")
parser.add_argument("-e", "--end", default=-1.0, type=float, help="end second")
parser.add_argument(
    "-b",
    "--bbox",
    default=[],
    nargs="+",
    type=int,
    help="if bbox='', then cut whole video, else cut the bbox, bbox format int: x y w h",
)

args = parser.parse_args()


def main() -> None:

    vid = VideoFileClip(args.input_file, audio=True)
    print(f"duration: {vid.duration}  width: {vid.w}   height: {vid.h}")

    if args.end == -1.0:
        end = vid.duration
    else:
        end = args.end
    start = args.start

    clip = vid.subclip(start, end)

    if args.bbox:
        x, y, w, h = args.bbox
        if (
            x + w <= vid.w
            and y + h <= vid.h
            and x < vid.w
            and y < vid.h
            and x >= 0
            and y >= 0
        ):
            clip = clip.crop(x1=int(x), y1=int(y), x2=int(x + w), y2=int(y + h))
        else:
            raise ValueError("bbox out of range")
    clip.write_videofile(args.output_file, audio_codec="aac")
    vid.close()
    print("Done!")


if __name__ == "__main__":
    main()



"""
@File    :   systeminfos.py
@Time    :   2024/06/26 14:17:46
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import List
import os
import os.path as osp
import time
import platform
from importlib import import_module


try:
    pynvml = import_module("pynvml")
except:
    os.system(f"{sys.executable} -m pip install nvidia-ml-py3")
    print("Installing nvidia-ml-py3,if failed,pip install nvidia-ml-py")
    pynvml = import_module("pynvml")


__all__ = ["general_info", "gpu_info"]


def general_info() -> None:
    print("#" * 25 + " System Info " + "#" * 25)
    print(f"platform:{platform.uname()}")
    print(f"cwd:{os.getcwd()}")
    print(f"(.):{osp.abspath('.')}")
    print(f"user:{osp.expanduser('~')}")

    try:
        s = "".join(os.popen("python -V").readlines()).strip()
    except:
        s = "No python"
    finally:
        print(f"{s}")

    s = "".join(os.popen(f"{sys.executable} -V").readlines())
    # print(f"{s}")
    print(f"ppath:{sys.executable}")
    print("#" * 25 + " CUDA Info " + "#" * 25)
    try:
        s = "".join(os.popen("nvcc -V").readlines()).strip()
    except:
        s = "No cuda"
    finally:
        print(f"{s}")


def get_gpu_device() -> List:
    deviceCount = pynvml.nvmlDeviceGetCount()
    gpu_list = []
    for i in range(deviceCount):
        handle = pynvml.nvmlDeviceGetHandleByIndex(i)
        print("GPU", i, ":", pynvml.nvmlDeviceGetName(handle))
        gpu_list.append(i)
    return gpu_list


def get_gpu_info(gpu_id: int) -> str:
    handle = pynvml.nvmlDeviceGetHandleByIndex(gpu_id)
    info = pynvml.nvmlDeviceGetMemoryInfo(handle)
    M = 1024**2
    gpu_info = "id:{}  total:{:.2f}M free:{:.2f}M  used:{:.2f}M free_rate:{}%".format(
        gpu_id, info.total / M, info.free / M, info.used / M, get_free_rate(gpu_id)
    )
    return gpu_info


def release() -> None:
    # 最后要关闭管理工具
    pynvml.nvmlShutdown()


def get_free_rate(gpu_id: int) -> int:
    handle = pynvml.nvmlDeviceGetHandleByIndex(gpu_id)
    info = pynvml.nvmlDeviceGetMemoryInfo(handle)
    free_rate = int((info.free / info.total) * 100)
    return free_rate


def gpu_info(keeptime: float = 0.001, interval_per_verbose: float = 0.2) -> None:
    """
    :param keeptime: minutes
    :return:
    """
    print("#" * 25 + " GPU Info " + "#" * 25)
    start = time.time()

    while True:
        pynvml.nvmlInit()
        gpu_devices = get_gpu_device()
        for gpuid in gpu_devices:
            print(get_gpu_info(gpuid))
        release()
        time.sleep(interval_per_verbose)
        if time.time() - start > 60 * keeptime:
            break


if __name__ == "__main__":
    general_info()
    gpu_info()



"""
@File    :   logger.py
@Time    :   2024/06/18 21:27:12
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Union, Optional, Dict
import os
import logging
from logging.handlers import TimedRotatingFileHandler

import colorlog


__all__ = ["Logger"]


streamFormatter = colorlog.ColoredFormatter(
    "%(log_color)s%(levelname)-9s%(asctime)s%(reset)s pid:%(process)d %(name)s line:%(lineno)s %(blue)s%(message)s",
    datefmt="%Y/%m/%d %H:%M:%S",
    reset=True,
    log_colors={
        "DEBUG": "cyan",
        "INFO": "green",
        "WARNING": "yellow",
        "ERROR": "red",
        "CRITICAL": "red,bg_white",
    },
    secondary_log_colors={},
    style="%",
)

fileFormatter = logging.Formatter(
    fmt="%(levelname)s %(asctime)s pid:%(process)d %(name)s line:%(lineno)s %(message)s",
    datefmt="%Y/%m/%d %H:%M:%S",
)


class Logger:

    WARNING = logging.WARNING
    ERROR = logging.ERROR
    CRITICAL = logging.CRITICAL
    INFO = logging.INFO
    DEBUG = logging.DEBUG

    def __init__(
        self,
        name: str,
        level: int = logging.DEBUG,
        log_filePath: str = "",
        is_saveLog: bool = False,
        when: str = "D",
        interval: int = 1,
        backupCount: int = 0,
    ) -> None:
        """
        Initialize a logger.

        Args:
            name (str): 日志名称.
            level (int): 日志级别，默认为 logging.DEBUG.
            log_filePath (str): 日志保存路径，如果为 None，则默认保存到工作目录下的 'logs' 文件夹中. Defaults to ''.
            is_saveLog (bool): 是否保存日志文件，如果为 True，则日志文件将保存到 log_filename 指定的路径. Defaults to False.
            when (str): 日志切割的时间单位，比如 'S'（秒）、'M'（分）、'H'（小时）、'D'（天）等. Defaults to "H".
            interval (int): 日志文件切割的时间间隔，例如当 when='H' 且 interval=1 时，表示每隔一个小时进行一次切割，并生成一个新的日志文件. Defaults to 1.
            backupCount (int): 定义默认保留旧日志文件的个数（如果超过这个数量，则会自动删除最早的日志文件），默认值为 0，表示不自动删除旧日志文件. Defaults to 0.

        Returns:
            None
        """
        self.log_filePath: Optional[str] = None
        self.curren_dir = os.getcwd()
        self.logger = logging.getLogger(name)
        self.logger.setLevel(level)
        if is_saveLog:
            os.makedirs(os.path.join(self.curren_dir, "logs"), exist_ok=True)
            if log_filePath == "":
                self.log_filePath = os.path.join(self.curren_dir, "logs", f"{name}.log")
            else:
                self.log_filePath = log_filePath
            fh = TimedRotatingFileHandler(
                self.log_filePath,
                when=when,
                interval=interval,
                backupCount=backupCount,
                encoding="utf-8",
            )

            # fh = logging.FileHandler(self.log_filePath,encoding="utf-8",filemode='w')
            fh.setLevel(level)
            fh.setFormatter(fileFormatter)
            self.logger.addHandler(fh)

        ch = logging.StreamHandler()
        ch.setLevel(level)
        ch.setFormatter(streamFormatter)
        self.logger.addHandler(ch)

    def __call__(self) -> logging.Logger:
        return self.logger
        
    def debug(self, msg: str) -> None:
        self.logger.debug(msg)

    def info(self, msg: str) -> None:
        self.logger.info(msg)

    def warning(self, msg: str) -> None:
        self.logger.warning(msg)

    def error(self, msg: str) -> None:
        self.logger.error(msg)

    def critical(self, msg: str) -> None:
        self.logger.critical(msg)


if __name__ == "__main__":
    logger = Logger("test", logging.DEBUG, is_saveLog=False)
    print(f'{logger:#}')
    logger.debug("debug")
    logger.info("info")
    logger.warning("warning")
    logger.error("error")
    logger.critical("critical")



import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Literal, Optional, Union

import os
import os.path as osp

__all__ = ["condarc", "pip", "cuda"]


def get_resource(
    path: str, returntype: Literal["content", "list"] = "content"
) -> Optional[str]:
    with open(path, "r", encoding="utf-8") as f:
        if returntype == "list":
            return f.readlines()
        elif returntype == "content":
            return f.read()
        else:
            return None


src_dir = osp.dirname(__file__)
condarc = get_resource(osp.join(src_dir, "condarc.txt"))
pip = get_resource(osp.join(src_dir, "pip.ini"))
cuda = get_resource(osp.join(src_dir, "cudarc.txt"))



# diarizarion.py

import os

from dotenv import load_dotenv
from pyannote.audio import Model, Pipeline, Inference
from pyannote.core import Segment, Annotation, Timeline
from scipy.spatial.distance import cosine
import torch
import torchaudio
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM
from transformers import AutoModelForSpeechSeq2Seq, AutoProcessor, pipeline

load_dotenv("../app_test/hf.env")
HUGGINGFACE_ACCESS_TOKEN = os.environ["HUGGINGFACE_ACCESS_TOKEN"]

model_id = "openai/whisper-large-v3-turbo"
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
PUNC_SENT_END = ['.', '?', '!']

torch_dtype = torch.float16 if torch.cuda.is_available() else torch.float32
# stt_model = WhisperModel(model_size, device=device, compute_type="float16")
stt_model = AutoModelForSpeechSeq2Seq.from_pretrained(
    model_id, torch_dtype=torch_dtype, low_cpu_mem_usage=True
)
stt_model.to(device)

processor = AutoProcessor.from_pretrained(model_id)

pipe = pipeline(
    "automatic-speech-recognition",
    model=stt_model,
    tokenizer=processor.tokenizer,
    feature_extractor=processor.feature_extractor,
    chunk_length_s=30,
    batch_size=16,  # batch size for inference - set based on your device
    torch_dtype=torch_dtype,
    device=device,
)

generate_kwargs={"language": "chinese",    # 语种 language
                 "logprob_threshold": -1.0,
                 "no_speech_threshold": 0.6,
                 }

def merge_cache(text_cache):
    sentence = ''.join([item[-1] for item in text_cache])
    spk = text_cache[0][1]
    start = text_cache[0][0].start
    end = text_cache[-1][0].end
    return Segment(start, end), spk, sentence

def get_text_with_timestamp(transcribe_res):
    timestamp_texts = []
    for item in transcribe_res['chunks']:
        start = item['timestamp'][0]
        end = item['timestamp'][1]
        text = item['text']
        timestamp_texts.append((Segment(start, end), text))
    return timestamp_texts

def add_speaker_info_to_text(timestamp_texts, ann):
    spk_text = []
    for seg, text in timestamp_texts:
        spk = ann.crop(seg).argmax()
        spk_text.append((seg, spk, text))
    return spk_text

def merge_sentence(spk_text):
    merged_spk_text = []
    pre_spk = None
    text_cache = []
    for seg, spk, text in spk_text:
        if spk != pre_spk and pre_spk is not None and len(text_cache) > 0:
            merged_spk_text.append(merge_cache(text_cache))
            text_cache = [(seg, spk, text)]
            pre_spk = spk

        elif text and len(text) > 0 and text[-1] in PUNC_SENT_END:
            text_cache.append((seg, spk, text))
            merged_spk_text.append(merge_cache(text_cache))
            text_cache = []
            pre_spk = spk
        else:
            text_cache.append((seg, spk, text))
            pre_spk = spk
    if len(text_cache) > 0:
        merged_spk_text.append(merge_cache(text_cache))
    return merged_spk_text

def diarize_text(transcribe_res, diarization_result):
    timestamp_texts = get_text_with_timestamp(transcribe_res)
    spk_text = add_speaker_info_to_text(timestamp_texts, diarization_result)
    res_processed = merge_sentence(spk_text)
    return res_processed


class Diarization:
    def __init__(self):
        self.pipeline = Pipeline.from_pretrained(
            "pyannote/speaker-diarization-3.1", use_auth_token=HUGGINGFACE_ACCESS_TOKEN
        ).to(device)

    def transform_diarization_output(self, diarization):
        l = []
        for segment, _, speaker in diarization.itertracks(yield_label=True):
            l.append({"start": segment.start, "end": segment.end, "speaker": speaker})
        return l

    def process(self, waveform, sample_rate):
        audio_tensor = waveform
        print(audio_tensor.shape)

        inference = self.pipeline(
            {
                "waveform": audio_tensor,
                "sample_rate": sample_rate,
                "min_speakers": 1,
                "max_speakers": 5,
            }
        )
        # convert output to list of dicts
        diarization = self.transform_diarization_output(inference)
        return inference, diarization


if __name__ == "__main__":
    target_sample_rate = 16000
    file_path=os.environ["file_path"]

    waveform, sample_rate = torchaudio.load(
        file_path
    )

    resampler = torchaudio.transforms.Resample(sample_rate, target_sample_rate) # 重采样
    waveform = resampler(torch.mean(waveform, dim=0, keepdim=True))   # torch.mean(waveform, dim=0, keepdim=True)用于torchaudio读的可能是多通道，转单通道

    result = pipe(waveform[0].numpy(), generate_kwargs=generate_kwargs,return_timestamps=True)
    print(result)
    # embed_model = Model.from_pretrained("pyannote/embedding", use_auth_token=HUGGINGFACE_ACCESS_TOKEN)
    diarization = Diarization()
    inference, diarization_result = diarization.process(waveform, target_sample_rate)
    print(diarization_result)

    res = diarize_text(result, inference)
    print(res)



"""
@File    :   register.py
@Time    :   2024/06/18 22:37:02
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Any, NoReturn, Union, Callable, Iterable, List, Dict, ValuesView, ItemsView, KeysView

from rlmc.utils.logger import Logger

__all__ = ["Register"]


logger = Logger(__name__, level=Logger.WARNING)


class Register:
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self._dict: Dict[str, Callable] = dict()

    def register(self, obj: Any) -> Callable:
        def add(key, val):
            if not callable(val):
                raise Exception(f"注册对象必须可调用，接收:{val}为非法对象")
            if key in self._dict:
                logger.warning(f"曾注册 {val.__name__}，现执行覆盖操作")
            self._dict[key] = val
            setattr(self, key, val)
            return val

        if callable(obj):
            return add(obj.__name__, obj)
        else:
            return lambda x: add(obj, x)

    def __call__(self, obj: Any) -> Callable:
        return self.register(obj)

    def __setitem__(self, key: str, val: Callable) -> None:
        self._dict[key] = val

    def __getitem__(self, key: str) -> Callable:
        return self._dict[key]

    def __contains__(self, key: Any) -> bool:
        return key in self._dict

    def __str__(self) -> str:
        return str(self._dict)

    def keys(self) -> KeysView:
        return self._dict.keys()

    def values(self) -> ValuesView:
        return self._dict.values()

    def items(self) -> ItemsView:
        return self._dict.items()

    def meta(self, mt_str: str) -> Callable:
        mt = self._dict[mt_str]
        if not isinstance(mt, type):
            raise TypeError("需要装饰元类对象，现有非法对象传入")

        def wrapper(cls):
            cls_ = mt(cls.__name__, cls.__bases__, {})
            return cls_

        return wrapper


if __name__ == "__main__":
    rg = Register()

    @rg("乘法")
    def mult(a: int, b: int):
        return a * b

    @rg
    def minus(a: int, b: int):
        return a - b

    print(rg)
    res = rg["乘法"](7, 2)
    print(rg.values())
    print(res)



# ref: https://blog.51cto.com/u_16213369/9371852

import argparse
import os
import time

import av
import cv2


parser = argparse.ArgumentParser("cut video")

parser.add_argument(
    "-i", "--input_file", default="", required=True, help="video file", type=str
)
parser.add_argument(
    "-o", "--output_file", default="./output.mp4", type=str, help="output file"
)
parser.add_argument("-s", "--start", default=0.0, type=float, help="start second")
parser.add_argument("-e", "--end", default=-1.0, type=float, help="end second")
parser.add_argument(
    "-b",
    "--bbox",
    default=[],
    nargs="+",
    type=int,
    help="if bbox='', then cut whole video, else cut the bbox, bbox format int: x y w h",
)

args = parser.parse_args()


def main() -> None:
    # 打开音视频文件
    container = av.open(args.input_file)

    cap = cv2.VideoCapture(args.input_file)
    total_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    fps = cap.get(cv2.CAP_PROP_FPS)
    frame_width = cap.get(cv2.CAP_PROP_FRAME_WIDTH)
    frame_height = cap.get(cv2.CAP_PROP_FRAME_HEIGHT)
    duration = total_frames / fps
    cap.release()

    print(
        f"total frames: {total_frames}, fps: {fps}, frame width: {frame_width}, frame height: {frame_height}, duration: {duration}"
    )

    # 获取音视频流
    video_stream = container.streams.video[0]
    audio_stream = container.streams.audio[0]

    layout = "stereo" if audio_stream.codec_context.channels > 1 else "mono"
    audio_sample_fmt = audio_stream.format.name
    audio_sample_rate = audio_stream.codec_context.sample_rate

    # 创建输出容器
    output_container = av.open(args.output_file, "w")

    # 添加音视频流到输出容器
    output_video_stream = output_container.add_stream(
        "h264", video_stream.codec_context.rate
    )
    output_audio_stream = output_container.add_stream(
        "aac", audio_stream.codec_context.rate
    )

    if args.bbox:
        x, y, w, h = args.bbox
        if (
            x + w <= frame_width
            and y + h <= frame_height
            and x < frame_width
            and y < frame_height
            and x >= 0
            and y >= 0
        ):
            output_video_stream.width = w
            output_video_stream.height = h
        else:
            raise ValueError("bbox out of range")

    # 设置剪辑的起始时间和结束时间
    start_time = args.start
    end_time = args.end

    for packet in container.demux():
        for frame in packet.decode():
            if packet.stream == video_stream:
                # 如果在剪辑的时间范围内，则编码并写入输出容器
                img_ndarray = frame.to_rgb().to_ndarray()

                if args.bbox:
                    img_ndarray = img_ndarray[y : y + h, x : x + w, :]

                new_frame = av.VideoFrame.from_ndarray(img_ndarray, format="rgb24")

                if end_time != -1.0:
                    if frame.time >= start_time and frame.time <= end_time:
                        output_packet = output_video_stream.encode(new_frame)
                        output_container.mux(output_packet)
                else:
                    if frame.time >= start_time:
                        output_packet = output_video_stream.encode(new_frame)
                        output_container.mux(output_packet)

            elif packet.stream == audio_stream:
                # 处理音频帧
                # 如果在剪辑的时间范围内，则编码并写入输出容器
                new_frame = av.AudioFrame.from_ndarray(
                    frame.to_ndarray(), format=audio_sample_fmt, layout=layout
                )
                new_frame.sample_rate = audio_sample_rate
                if end_time != -1.0:
                    if frame.time >= start_time and frame.time <= end_time:
                        output_packet = output_audio_stream.encode(new_frame)
                        output_container.mux(output_packet)
                else:
                    if frame.time >= start_time:
                        output_packet = output_audio_stream.encode(new_frame)
                        output_container.mux(output_packet)

    output_container.close()


if __name__ == "__main__":
    main()



"""
@File    :   mic.py
@Time    :   2024/07/04 12:21:43
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

import multiprocessing as mp
import os
import os.path as osp
import time
import copy
import argparse
from typing import Callable, Optional, Union

import pynput
import pyaudio
import wave
import numpy as np


parser = argparse.ArgumentParser("microphone")
parser.add_argument(
    "-d",
    "--save_dir",
    required=True,
)
args = parser.parse_args()

INT16_MAX_ABS_VALUE = 32768.0
CHUNK = 512
FORMAT = pyaudio.paInt16
CHANNELS = 1
RATE = 16000
unit = RATE // CHUNK


def on_press_wrapper(signal: mp.Value) -> Callable:
    def on_press(key: pynput.keyboard.Key) -> Optional[bool]:
        nonlocal signal
        if key == pynput.keyboard.Key.esc:
            signal.value = -1
            return False
        elif hasattr(key, "char"):
            if key.char == "f":
                print("start record...")
                signal.value = 1
            if key.char == "s":
                print("end record...")
                signal.value = 0

    return on_press
    # print("Key {0} pressed".format(key))


def kb_listener(signal: mp.Value) -> None:
    listener = pynput.keyboard.Listener(on_press=on_press_wrapper(signal))
    listener.start()
    listener.join()


def main() -> None:
    p = pyaudio.PyAudio()
    # 打开麦克风流
    stream = p.open(
        format=FORMAT,
        channels=CHANNELS,
        rate=RATE,
        input=True,
        output=True,
        frames_per_buffer=CHUNK,
    )
    audio_collector = []

    signal = mp.Value("i", 0)
    listener = mp.Process(target=kb_listener, args=(signal,))
    listener.daemon = True
    daytime_start = time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime(time.time()))
    print("按下F开始录音，按下S暂停录音，按下ESC退出程序")
    print("ready")
    listener.start()

    while True:
        if signal.value == 1:
            audio_data = stream.read(CHUNK)
            audio_collector.append(audio_data)

        elif signal.value == -1:
            break
        else:
            pass
        # time.sleep(0.2)
    # audio_array = np.frombuffer(b"".join(audio_collector), dtype=np.int16)
    # audio_array = audio_array.astype(np.float32) / INT16_MAX_ABS_VALUE

    listener.join()

    stream.stop_stream()
    stream.close()
    p.terminate()

    daytime_end = time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime(time.time()))
    wf = wave.open(
        os.path.join(args.save_dir, f"{daytime_start}__{daytime_end}.wav"), "wb"
    )
    # 设置音频的声道数、采样宽度和采样率
    wf.setnchannels(1)
    wf.setsampwidth(p.get_sample_size(pyaudio.paInt16))
    wf.setframerate(16000)
    # 将 frames 列表中的所有数据连接成一个字节串，并写入文件
    wf.writeframes(b"".join(audio_collector))
    # 关闭文件
    wf.close()


if __name__ == "__main__":
    main()



"""
@File    :   trainer.py
@Time    :   2024/06/30 00:24:23
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Union, Dict, List, Tuple
import time
import os
import os.path as osp

from tqdm import tqdm
from PIL import Image
import numpy as np
import torch
from torch.nn import functional as F
import torchvision
from torchvision import datasets, transforms
from torchvision.transforms import (
    Compose,
    Resize,
    ToTensor,
    Normalize,
    InterpolationMode,
)
import thop
from torchsummary import summary

from rlmc.fileop.yaml_op import Yaml
from rlmc.utils.logger import Logger
from rlmc.trainer.loss import loss_method
from rlmc.trainer.opt import optimizer
from rlmc.data.utils import onehot2mask
from rlmc.trainer.met import metrics
from sklearn.metrics import precision_score, recall_score, f1_score, accuracy_score


__all__ = ["Trainer", "Predict"]


logger = Logger(__name__, level=Logger.INFO)


class BaseTrainer:
    def __init__(self): ...

    def model_summary(self) -> None:
        print("model summary:")
        summary(
            self.model,
            (
                self.args.model.in_channels,
                self.args.dataset.input_size.height,
                self.args.dataset.input_size.width,
            ),
        )
        input = torch.randn(
            1,
            self.args.model.in_channels,
            self.args.dataset.input_size.height,
            self.args.dataset.input_size.width,
        ).to(self.device)
        macs, params = thop.profile(self.model, inputs=(input,))
        print("FLOPs:", macs)  # FLOPs模型复杂度
        print("params:", params)  # params参数量
        macs, params = thop.clever_format([macs, params], "%.3f")
        print("FLOPs:", macs)
        print("params:", params)


class Trainer(BaseTrainer):
    def __init__(
        self,
        configs: Dict,
        model: torch.nn.Module,
        train_dataset: torch.utils.data.Dataset,
        val_dataset: torch.utils.data.Dataset,
        device: str,
    ) -> None:
        self.args = configs
        self.model = model.to(device)
        timestamp = time.time()
        self.log_time = time.strftime("%Y_%m_%d_%H_%M", time.localtime(timestamp))
        self.ckpt_dir = osp.join(self.args.train.save_dir, self.log_time)
        os.makedirs(self.ckpt_dir, exist_ok=True)
        Yaml().write(self.args.to_dict(), osp.join(self.ckpt_dir, "log_configs.yaml"))
        self.train_loader = torch.utils.data.DataLoader(
            train_dataset,
            batch_size=self.args.train.dataloader.batch_size,
            shuffle=self.args.train.dataloader.shuffle,
            num_workers=self.args.train.dataloader.num_workers,
            pin_memory=self.args.train.dataloader.pin_memory,
            drop_last=self.args.train.dataloader.drop_last,
        )

        self.val_loader = torch.utils.data.DataLoader(
            val_dataset,
            batch_size=self.args.eval.dataloader.batch_size,
            shuffle=self.args.eval.dataloader.shuffle,
            num_workers=self.args.eval.dataloader.num_workers,
            pin_memory=self.args.eval.dataloader.pin_memory,
            drop_last=self.args.eval.dataloader.drop_last,
        )

        self.device = device
        self.criterion = loss_method[self.args.train.loss]
        self.optimizer = optimizer[self.args.train.optimizer.name](
            self.model.parameters(),
            lr=self.args.train.optimizer.lr,
            weight_decay=self.args.train.optimizer.weight_decay,
        )

        self.metrics = metrics[self.args.train.metrics](self.args.dataset.num_classes)

        self.scheduler = torch.optim.lr_scheduler.MultiStepLR(
            self.optimizer,
            self.args.train.optimizer.milestones,
            gamma=self.args.train.optimizer.gamma,
            last_epoch=-1,
        )
        if self.args.train.show_model_summary:
            self.model_summary()

    def train(self) -> None:
        for epoch in range(self.args.train.max_epochs):
            self.model.train()
            for batch_idx, (data, target) in enumerate(self.train_loader):
                data, target = data.to(self.device), target.to(self.device)

                self.optimizer.zero_grad()
                output = self.model(data)

                # metric_pred=output.clone().detach().cpu()
                # metric_tgt=target.clone().detach().view(-1).cpu().numpy().astype(np.int8)
                # metric_pred=torch.unsqueeze(torch.argmax(metric_pred,dim=1),dim=1)
                # tgt_shape=target.shape
                # mask=torch.zeros(tgt_shape[0],tgt_shape[1],tgt_shape[2],tgt_shape[3])
                # mask=mask.scatter(1, metric_pred,1).view(-1).numpy().astype(np.int8)

                ########### metrics ############
                metric_pred = output.clone().detach()
                metric_pred = torch.argmax(metric_pred, dim=1).view(-1).cpu().numpy()
                metric_tgt = target.clone().detach()
                metric_tgt = torch.argmax(metric_tgt, dim=1).view(-1).cpu().numpy()
                self.metrics.addBatch(metric_pred, metric_tgt)
                ########### metrics ############

                loss = self.criterion(output, target)
                loss.backward()
                self.optimizer.step()
                self.scheduler.step()

                if batch_idx % self.args.train.log_interval == 0:
                    # accuracy=accuracy_score(metric_tgt,mask)
                    # precision=precision_score(metric_tgt,mask)
                    # recall=recall_score(metric_tgt,mask)
                    # f1=f1_score(metric_tgt,mask)
                    # logger.info(
                    #     f"Train Epoch: {epoch} [{batch_idx}/{len(self.train_loader)} "
                    #     + f"({100. * batch_idx / len(self.train_loader):.2f}%)]\tLoss: {loss.item():.6f}\tAcc: {accuracy}\tPrecision: {precision}\tRecall: {recall}\tf1: {f1}"
                    # )

                    ########### metrics ############
                    acc = self.metrics.pixelAccuracy()
                    macc = self.metrics.meanPixelAccuracy()
                    mIoU = self.metrics.meanIntersectionOverUnion()
                    fwIoU = self.metrics.Frequency_Weighted_Intersection_over_Union()
                    self.metrics.reset()
                    ########### metrics ############

                    logger.info(
                        f"Train Epoch: {epoch} [{batch_idx}/{len(self.train_loader)}] "
                        + " iter_progress={:.2f}%  loss={:.6f}, acc={:.2f}%, macc={:.2f}%, mIoU={:.2f}%, fwIoU={:.2f}%".format(
                            100 * batch_idx / len(self.train_loader),
                            loss.item(),
                            acc * 100,
                            macc * 100,
                            mIoU * 100,
                            fwIoU * 100,
                        )
                    )
            if self.args.train.is_eval and epoch != 0:
                if (
                    epoch % self.args.train.eval_interval == 0
                ) or epoch == self.args.train.max_epochs - 1:
                    self.evaluate(epoch, is_training=True)

    def evaluate(self, epoch: int = 10000000, is_training: bool = False) -> None:
        if not is_training:
            state_dict = torch.load(self.args.eval.restore_model_path)
            self.model.load_state_dict(state_dict, strict=True)
        self.model.eval()
        with torch.no_grad():
            for val_batch_idx, (val_data, val_target) in enumerate(self.val_loader):
                val_data, val_target = val_data.to(self.device), val_target.to(
                    self.device
                )
                val_output = self.model(val_data)

                ########### metrics ############
                metric_pred = val_output.clone().detach()
                metric_pred = torch.argmax(metric_pred, dim=1).view(-1).cpu().numpy()
                metric_tgt = val_target.clone().detach()
                metric_tgt = torch.argmax(metric_tgt, dim=1).view(-1).cpu().numpy()
                self.metrics.addBatch(metric_pred, metric_tgt)
                ########### metrics ############

                val_loss = self.criterion(val_output, val_target)

        if is_training:
            ########### metrics ############
            acc = self.metrics.pixelAccuracy()
            macc = self.metrics.meanPixelAccuracy()
            mIoU = self.metrics.meanIntersectionOverUnion()
            fwIoU = self.metrics.Frequency_Weighted_Intersection_over_Union()
            self.metrics.reset()
            ########### metrics ############
            logger.info(
                f"Val Epoch: {epoch} [{val_batch_idx}/{len(self.val_loader)}] "
                + " iter_progress={:.2f}%  loss={:.6f}, acc={:.2f}%, macc={:.2f}%, mIoU={:.2f}%, fwIoU={:.2f}%".format(
                    100 * val_batch_idx / len(self.val_loader),
                    val_loss.item(),
                    acc * 100,
                    macc * 100,
                    mIoU * 100,
                    fwIoU * 100,
                )
            )

            keys = []
            state_dict = self.model.state_dict()
            for k, v in state_dict.items():
                if (
                    "total_" in k
                ):  # 将torchsummary或thop生成的含total_的Unexpected key过滤掉，避免eval或predict设置为strict=True时读取模型state_dict报错
                    continue
                keys.append(k)

            new_dict = {k: state_dict[k] for k in keys}

            torch.save(
                new_dict,
                osp.join(self.ckpt_dir, f"model_{epoch:04d}.pth"),
            )
        else:
            logger.info(f"Val Loss: {val_loss.item():.6f}")


class Predict(BaseTrainer):
    def __init__(self, configs: Dict, model: torch.nn.Module, device: str) -> None:
        self.args = configs
        self.model = model
        self.device = device
        self.predict_transform = Compose(
            [
                Resize(
                    (
                        self.args.dataset.input_size.height,
                        self.args.dataset.input_size.width,
                    ),
                    interpolation=InterpolationMode.BICUBIC,
                ),
                ToTensor(),
                Normalize(
                    self.args.dataset.transformNormalize.mean,
                    self.args.dataset.transformNormalize.std,
                ),
            ]
        )
        os.makedirs(self.args.predict.save_dir, exist_ok=True)
        if self.args.predict.model_summary:
            self.model_summary()
        state_dict = torch.load(self.args.predict.restore_model_path)
        self.model.load_state_dict(state_dict, strict=True)
        self.model.to(self.device)
        self.model.eval()
        self.palette = np.array(
            [
                [0, 0, 0],
                [128, 0, 0],
                [0, 128, 0],
                [128, 128, 0],
                [0, 0, 128],
                [128, 0, 128],
                [0, 128, 128],
                [128, 128, 128],
                [64, 0, 0],
                [192, 0, 0],
                [64, 128, 0],
                [192, 128, 0],
                [64, 0, 128],
                [192, 0, 128],
                [64, 128, 128],
                [192, 128, 128],
                [0, 64, 0],
                [128, 64, 0],
                [0, 192, 0],
                [128, 192, 0],
                [0, 64, 128],
            ]
        ).astype(np.uint8)

    def predict(
        self, img_path: str, is_save: bool = False, is_show: bool = True
    ) -> None:
        filename = osp.splitext(osp.basename(img_path))[0]
        with torch.no_grad():
            img = Image.open(img_path)
            width, height = img.size
            img = self.predict_transform(img).unsqueeze(0).to(self.device)
            output = self.model(img)

            output = onehot2mask(output, axis=1)
            output = output.to("cpu")[0].numpy().astype(np.uint8)

            output = Image.fromarray(output)
            output = output.resize((width, height), Image.NEAREST)

            print("output", output)
            #    output[output>class_num]=0
            output.putpalette(self.palette)
            if is_save:
                output.save(osp.join(self.args.predict.save_dir, filename + ".png"))

            if is_show:
                output.show()



'''
@File    :   abstract_file.py
@Time    :   2024/06/21 20:46:38
@Author  :   RayLam
@Contact :   1027196450@qq.com
'''
from abc import ABC, abstractmethod

class AbstractFile(ABC):
    @abstractmethod
    def read(self):
        pass

    @abstractmethod
    def write(self):
        pass


    



import argparse
import os
import os.path as osp
import sqlite3
import hashlib
from typing import List

from PIL import ImageTk, Image
import numpy as np
import torch
import cv2
from tqdm import tqdm
import clip
import faiss

os.environ["KMP_DUPLICATE_LIB_OK"] = "TRUE"

parser = argparse.ArgumentParser("image_search")
parser.add_argument(
    "-r",
    "--img_root",
    required=True,
)
parser.add_argument(
    "-t",
    "--text_for_matching",
    required=True,
)
parser.add_argument(
    "-a",
    "--alternative",  # 备选结果数
    default=5,
)

parser.add_argument(
    "-f",
    "--is_rebuild_faiss_index",
    action="store_true",
    default=False,
)


args = parser.parse_args()


device = "cuda" if torch.cuda.is_available() else "cpu"
model, preprocess = clip.load("ViT-B/32", device=device)


def generate_id(string: str):
    sha256 = hashlib.sha256()
    sha256.update(string.encode("utf-8"))
    img_id = int.from_bytes(sha256.digest()[:6], byteorder="big")
    return img_id


def clipv(root: str):
    dimension = 512
    idx_flat = faiss.IndexFlatL2(dimension)
    index = faiss.IndexIDMap(idx_flat)

    faiss_idx_file = osp.join(root, "faissidx.idx")
    db = osp.join(root, "imgsnames.db")
    img_paths = find_files_with_ext(root)

    if osp.exists(faiss_idx_file):
        index = faiss.read_index(faiss_idx_file)
    conn = sqlite3.connect(db)
    cursor = conn.cursor()
    cursor.execute(
        """CREATE TABLE IF NOT EXISTS images (id INTEGER PRIMARY KEY, name TEXT)"""
    )
    conn.commit()

    exist_ids = set(row[0] for row in cursor.execute("SELECT id FROM images"))
    collector = []
    for img_path in img_paths:
        image_id = generate_id(img_path)
        if image_id not in exist_ids:
            collector.append(img_path)

    counter = 1
    for img_path in collector:

        img_id = generate_id(img_path)
        img = preprocess(Image.open(img_path)).unsqueeze(0).to(device)
        with torch.no_grad():
            img_feat = model.encode_image(img)
        img_feat_np = img_feat.cpu().numpy()
        norms = np.linalg.norm(img_feat_np, axis=1, keepdims=True)
        img_feat_np /= norms
        index.add_with_ids(img_feat_np, np.array([img_id]))
        try:
            cursor.execute(
                "INSERT INTO images (id, name) VALUES (?, ?)", (img_id, img_path)
            )
        except Exception as e:
            print(f"Error {img_path}: {str(e)}")
        if counter % 200 == 0:
            conn.commit()
            faiss.write_index(index, faiss_idx_file)
        counter += 1

    conn.commit()
    faiss.write_index(index, faiss_idx_file)
    print("clip vectorized." + "\n")


def search_vec(text: str, root: str, alternative: int = 5):

    text_tokenized = clip.tokenize([text]).to(device)
    with torch.no_grad():
        text_feat = model.encode_text(text_tokenized)
    text_feat_np = text_feat.cpu().numpy()
    faiss_index_file = os.path.join(root, "faissidx.idx")
    index = faiss.read_index(faiss_index_file)
    norm = np.linalg.norm(text_feat_np, axis=1, keepdims=True)
    text_feat_np /= norm

    db = os.path.join(root, "imgsnames.db")
    D, I = index.search(text_feat_np, alternative)
    img_paths = []
    conn = sqlite3.connect(db)
    cursor = conn.cursor()
    for i, idx in enumerate(I[0]):
        img_id = idx
        cursor.execute("SELECT name FROM images WHERE id = ?", (str(img_id),))
        res = cursor.fetchone()
        if res:
            img_name = res[0]
            img_paths.append(str(img_name))
        else:
            print(f"{i+1}: Image not found(ID: {img_id}, Distance: {D[0][i]})")
    conn.close()
    print(f"Results: {img_paths}")


def find_files_with_ext(
    folder_root: str,
    extensions: List[str] = [
        ".PNG",
        ".JPG",
        ".JPEG",
        ".png",
        ".jpg",
        ".jpeg",
        ".gif",
        ".webp",
    ],
):
    file_list = []

    for root, dirs, files in os.walk(folder_root):
        for file in files:
            if osp.splitext(file)[-1] in extensions:
                file_list.append(os.path.join(root, file))

    return file_list


def main() -> None:
    if args.is_rebuild_faiss_index:
        try:
            os.remove(args.img_root + "/faissidx.idx")
        except:
            pass
        try:
            os.remove(args.img_root + "/imgsnames.db")
        except:
            pass
        clipv(args.img_root)
    else:
        if not osp.exists(args.img_root + "/faissidx.idx") or not osp.exists(
            args.img_root + "/imgsnames.db"
        ):
            clipv(args.img_root)
        else:
            ...
    search_vec(args.text_for_matching, args.img_root, args.alternative)


if __name__ == "__main__":
    main()



"""
@File    :   __init__.py
@Time    :   2024/06/30 00:25:19
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")


__all__ = ["predictors", "models"]

from rlmc.model.cv.semantic_segmentation import SemanticSegmentationModel

from rlmc.model.predictor import (
    ClsPredictor,
    SemanticSegmentationPredictor,
    PosePredictor,
    ObjDetectPredictor,
    RegressorPredictor,
    SequencePredictor,
)

models = {"seg": SemanticSegmentationModel}

predictors = {
    "cls": ClsPredictor,
    "seg": SemanticSegmentationPredictor,
    "pose": PosePredictor,
    "det": ObjDetectPredictor,
    "reg": RegressorPredictor,
    "seq": SequencePredictor,
}



import hashlib
import argparse

import rich.progress

parser = argparse.ArgumentParser("generate hash")
parser.add_argument("-f", "--file", required=True)
parser.add_argument("-a", "--algorithm", default="md5", help="md5, sha1, sha256")
args = parser.parse_args()


def encrypt(fpath: str, algorithm: str) -> str:
    with rich.progress.open(fpath, "rb") as f:
        hash = hashlib.new(algorithm)
        for chunk in iter(lambda: f.read(2**20), b""):
            hash.update(chunk)
        return hash.hexdigest()


def gen_hash() -> None:
    hexdigest = encrypt(args.file, args.algorithm)
    print(f"{args.algorithm}: {hexdigest}")


if __name__ == "__main__":
    gen_hash()
    # for algorithm in ('md5', 'sha1', 'sha256'):
    #     hexdigest = encrypt(r"C:\Users\\Desktop\git_update\git_update.bat", algorithm)
    #     print(f'{algorithm}: {hexdigest}')



"""
@File    :   utils.py
@Time    :   2024/06/25 10:36:13
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Union, List, Dict
import os
import os.path as osp
import glob
import shutil
import random
import re

import numpy as np
from tqdm import tqdm

__all__ = ["DatasetSplit", "FindContent"]


class DatasetSplit:
    """
    if __name__ == "__main__":
        ds = DatasetSplit(
            "./imgs",
            "./labels",
            img_exts=["*.jpg"],
            label_ext="*.xml",
            split={"train": 0.8, "val": 0.1, "test": 0.1},
            is_split_name_up=False,
        )
        ds.process()
    """

    def __init__(
        self,
        img_dir: str,
        label_dir: str,
        img_exts: List[str] = ["*.jpg"],
        label_ext: str = "*.png",
        split: Dict[str, float] = {"train": 0.8, "val": 0.1, "test": 0.1},
        is_shuffle: bool = True,
        random_seed: int = 123,
        is_split_name_up: bool = True,
    ) -> None:
        if sum(split.values()) != 1:
            raise ValueError("split values must sum to 1")
        self.img_dir = img_dir
        self.label_dir = label_dir
        self.img_files = self._collect_files(img_dir, img_exts)
        self.label_ext = label_ext.split(".")[-1]
        self.output_dir = osp.dirname(self.img_dir) + "/processed"
        self.split = split
        self.is_split_name_up = is_split_name_up
        self.pairs = self._pair_files()
        print(self.pairs)
        if is_shuffle:
            random.seed(random_seed)
            random.shuffle(self.pairs)

    def _collect_files(
        self, directory: str, exts: List[str] = ["*.jpg", "*.png"]
    ) -> List[str]:
        files = []
        for ext in exts:
            files.extend(glob.glob(osp.join(directory, ext)))
        return files

    def _pair_files(self) -> List:
        pairs = []
        for img_file in self.img_files:
            label_file = osp.join(
                self.label_dir,
                osp.splitext(osp.basename(img_file))[0] + "." + self.label_ext,
            )
            if osp.isfile(label_file):
                pairs.append((img_file, label_file))
        return pairs

    def _split_files(self) -> None:
        img_lab = ["images", "labels"]  # 可改此处改目录文件夹名

        lenth = len(self.pairs)
        counter = 0

        for split_idx, (k, v) in enumerate(self.split.items()):
            print("################### processing: ", k, " #####################")
            num = int(lenth * v)
            if split_idx == len(self.split) - 1:
                num = lenth - counter
            assert num > 1, f"{k}数据过少，请检查数据集"

            for i in tqdm(range(num)):
                img_file, label_file = self.pairs[counter]
                if self.is_split_name_up:
                    img_tgt = osp.join(
                        self.output_dir, k, img_lab[0], osp.basename(img_file)
                    )
                    label_tgt = osp.join(
                        self.output_dir, k, img_lab[1], osp.basename(label_file)
                    )
                else:
                    img_tgt = osp.join(
                        self.output_dir, img_lab[0], k, osp.basename(img_file)
                    )
                    label_tgt = osp.join(
                        self.output_dir, img_lab[1], k, osp.basename(label_file)
                    )
                os.makedirs(osp.dirname(img_tgt), exist_ok=True)
                os.makedirs(osp.dirname(label_tgt), exist_ok=True)

                shutil.copy(img_file, img_tgt)
                shutil.copy(label_file, label_tgt)
                counter += 1
        print("counter:", counter, "  data_total_lenth:", lenth)

    def process(self) -> None:
        self._split_files()
        print("Done!")


class FindContent:
    """
    if __name__ == "__main__":
        fc=FindContent(
            content=" $waffa",
            file_dir="./nginx1-7-11-3Gryphon",
            exts=['*.conf','*.py', '*.exe'], encoding='utf-8',is_use_regex=False, is_recursive=True
        )
        res=fc.process()
        print(res)
    """

    def __init__(
        self,
        content: str,
        file_dir: str,
        exts: List[str] = ["*.conf", "*.py"],
        encoding: str = "utf-8",
        is_use_regex: bool = False,
        is_recursive: bool = True,
    ) -> None:
        self.content = content
        self.file_dir = file_dir
        self.exts = exts
        self.encoding = encoding
        self.is_use_regex = is_use_regex
        self.is_recursive = is_recursive

    def find_string_in_file(self, file_path: str, search_string: str) -> bool:
        with open(file_path, "r", encoding=self.encoding) as file:
            for line in file.readlines():
                if self.is_use_regex:
                    if re.search(search_string, line):
                        return True
                else:
                    if search_string in line:
                        return True
        return False

    def process(self) -> List[str]:
        files = []
        for ext in self.exts:
            if self.is_recursive:
                files.extend(
                    glob.glob(
                        osp.join(self.file_dir, "**", ext), recursive=self.is_recursive
                    )
                )
            else:
                files.extend(
                    glob.glob(osp.join(self.file_dir, ext), recursive=self.is_recursive)
                )
        print(files)
        collector = []
        for file in tqdm(files):
            try:
                is_found = self.find_string_in_file(file, self.content)
            except UnicodeDecodeError as ue:
                print(f"\nskip:UnicodeDecodeError processing file {file}: {ue}")
                is_found = False
            except Exception as e:
                print(f"\nskip:Error processing file {file}: {e}")
                is_found = False
            if is_found:
                collector.append(file)

        return collector


if __name__ == "__main__":
    # ds = DatasetSplit(
    #     "./imgs",
    #     "./labels",
    #     img_exts=["*.jpg"],
    #     label_ext="*.xml",
    #     split={"train": 0.8, "val": 0.1, "test": 0.1},
    #     is_split_name_up=False,
    # )
    # ds.process()

    fc = FindContent(
        content=".*waffable.*",  # ".* waffa.*",
        file_dir=r"",
        exts=["*.conf", "*.py", "*.exe"],
        encoding="utf-8",
        is_use_regex=True,
        is_recursive=False,
    )
    res = fc.process()
    print(res)



"""
https://blog.csdn.net/sinat_29047129/article/details/103642140
https://www.cnblogs.com/Trevo/p/11795503.html
refer to https://github.com/jfzhang95/pytorch-deeplab-xception/blob/master/utils/metrics.py
"""

import numpy as np

__all__ = ["metrics"]


class SegmentationMetric:
    def __init__(self, numClass: int) -> None:
        self.numClass = numClass
        self.confusionMatrix = np.zeros((self.numClass,) * 2)  # 混淆矩阵n*n，初始值全0

    # 像素准确率PA，预测正确的像素/总像素
    def pixelAccuracy(self) -> float:
        # return all class overall pixel accuracy
        # acc = (TP + TN) / (TP + TN + FP + TN)
        acc = np.diag(self.confusionMatrix).sum() / self.confusionMatrix.sum()
        return acc

    # 类别像素准确率CPA，返回n*1的值，代表每一类，包括背景
    def classPixelAccuracy(self):
        # return each category pixel accuracy(A more accurate way to call it precision)
        # acc = (TP) / TP + FP
        classAcc = np.diag(self.confusionMatrix) / np.maximum(
            self.confusionMatrix.sum(axis=1), 1
        )
        return classAcc

    # 类别平均像素准确率MPA，对每一类的像素准确率求平均
    def meanPixelAccuracy(self):
        classAcc = self.classPixelAccuracy()
        meanAcc = np.nanmean(classAcc)
        return meanAcc

    # MIoU
    def meanIntersectionOverUnion(self) -> float:
        # Intersection = TP Union = TP + FP + FN
        # IoU = TP / (TP + FP + FN)
        intersection = np.diag(self.confusionMatrix)
        union = np.maximum(
            np.sum(self.confusionMatrix, axis=1)
            + np.sum(self.confusionMatrix, axis=0)
            - np.diag(self.confusionMatrix),
            1,
        )
        IoU = intersection / union
        mIoU = np.nanmean(IoU)
        return mIoU

    # 根据标签和预测图片返回其混淆矩阵
    def genConfusionMatrix(self, imgPredict: np.ndarray, imgLabel: np.ndarray):
        # remove classes from unlabeled pixels in gt image and predict
        mask = (imgLabel >= 0) & (imgLabel < self.numClass)
        label = self.numClass * imgLabel[mask].astype(int) + imgPredict[mask]
        count = np.bincount(label, minlength=self.numClass**2)
        confusionMatrix = count.reshape(self.numClass, self.numClass)
        return confusionMatrix

    def Frequency_Weighted_Intersection_over_Union(self) -> float:
        # FWIOU = [(TP+FN)/(TP+FP+TN+FN)] *[TP / (TP + FP + FN)]
        freq = np.sum(self.confusionMatrix, axis=1) / np.sum(self.confusionMatrix)
        iu = np.diag(self.confusionMatrix) / (
            np.sum(self.confusionMatrix, axis=1)
            + np.sum(self.confusionMatrix, axis=0)
            - np.diag(self.confusionMatrix)
        )
        FWIoU = (freq[freq > 0] * iu[freq > 0]).sum()
        return FWIoU

    def addBatch(self, imgPredict: np.ndarray, imgLabel: np.ndarray) -> None:
        assert imgPredict.shape == imgLabel.shape  # 确认标签和预测值图片大小相等
        self.confusionMatrix += self.genConfusionMatrix(imgPredict, imgLabel)

    def reset(self) -> None:
        self.confusionMatrix = np.zeros((self.numClass, self.numClass))


metrics = {"SegmetationMetric": SegmentationMetric}



"""
@File    :   stt.py
@Time    :   2024/07/10 09:57:46
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from typing import Optional, Union, List, Dict
import argparse
import time

import numpy as np
import torch
import pyaudio
import pynput
import pyperclip
# from faster_whisper import WhisperModel
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM
from transformers import AutoModelForSpeechSeq2Seq, AutoProcessor, pipeline


parser = argparse.ArgumentParser("stt")
parser.add_argument(
    "-r",
    "--record_time",
    default=5,
    type=int,
    help="record time(second)",
)
parser.add_argument(
    "-l",
    "--record_src_language",
    default="zh",
    help="record src language",
)

parser.add_argument("-i", "--is_translate", action="store_true", help="is_translate")
parser.add_argument(
    "-s",
    "--translate_src_lang",
    default="zho_Hans",
    help="translate zho_Hans, zho_Hant,eng_Latn, jpn_Jpan,yue_Hant,kor_Hang",
)
parser.add_argument(
    "-t",
    "--translate_tgt_lang",
    default="eng_Latn",
    help="translate zho_Hans, zho_Hant,eng_Latn, jpn_Jpan,yue_Hant,kor_Hang",  # zho_Hant繁体
)
parser.add_argument(
    "-m",
    "--translate_model",
    default="facebook/nllb-200-distilled-600M",
    help="translate model",
)
args = parser.parse_args()

signal = 0
INT16_MAX_ABS_VALUE = 32768.0
CHUNK = 512
FORMAT = pyaudio.paInt16
CHANNELS = 1
RATE = 16000


model_id = "openai/whisper-large-v3-turbo"
device = "cuda" if torch.cuda.is_available() else "cpu"
torch_dtype = torch.float16 if torch.cuda.is_available() else torch.float32
# stt_model = WhisperModel(model_size, device=device, compute_type="float16")
stt_model = AutoModelForSpeechSeq2Seq.from_pretrained(
    model_id, torch_dtype=torch_dtype, low_cpu_mem_usage=True
)
stt_model.to(device)

processor = AutoProcessor.from_pretrained(model_id)

pipe = pipeline(
    "automatic-speech-recognition",
    model=stt_model,
    tokenizer=processor.tokenizer,
    feature_extractor=processor.feature_extractor,
    chunk_length_s=30,
    batch_size=16,  # batch size for inference - set based on your device
    torch_dtype=torch_dtype,
    device=device,
)

generate_kwargs={"language": "chinese",    
                 "logprob_threshold": -1.0,
                 "no_speech_threshold": 0.6,
                 "return_timestamps": False,}

# language:
#   - en
#   - zh
#   - de
#   - es
#   - ru
#   - ko
#   - fr
#   - ja
#   - pt
#   - tr
#   - pl
#   - ca
#   - nl
#   - ar
#   - sv
#   - it
#   - id
#   - hi
#   - fi
#   - vi
#   - he
#   - uk
#   - el
#   - ms
#   - cs
#   - ro
#   - da
#   - hu
#   - ta
#   - 'no'
#   - th
#   - ur
#   - hr
#   - bg
#   - lt
#   - la
#   - mi
#   - ml
#   - cy
#   - sk
#   - te
#   - fa
#   - lv
#   - bn
#   - sr
#   - az
#   - sl
#   - kn
#   - et
#   - mk
#   - br
#   - eu
#   - is
#   - hy
#   - ne
#   - mn
#   - bs
#   - kk
#   - sq
#   - sw
#   - gl
#   - mr
#   - pa
#   - si
#   - km
#   - sn
#   - yo
#   - so
#   - af
#   - oc
#   - ka
#   - be
#   - tg
#   - sd
#   - gu
#   - am
#   - yi
#   - lo
#   - uz
#   - fo
#   - ht
#   - ps
#   - tk
#   - nn
#   - mt
#   - sa
#   - lb
#   - my
#   - bo
#   - tl
#   - mg
#   - as
#   - tt
#   - haw
#   - ln
#   - ha
#   - ba
#   - jw
#   - su

def on_press(key: pynput.keyboard.Key) -> Optional[bool]:
    if key == pynput.keyboard.Key.esc:
        global signal
        signal = 1
        return False
    elif hasattr(key, "char"):
        if key.char == "f":
            return False
    # print("Key {0} pressed".format(key))


def kb_listener() -> None:
    listener = pynput.keyboard.Listener(on_press=on_press)
    listener.start()
    listener.join()


def speech_to_text() -> str:
    # 创建一个PyAudio对象
    p = pyaudio.PyAudio()

    # 打开麦克风流
    stream = p.open(
        format=FORMAT,
        channels=CHANNELS,
        rate=RATE,
        input=True,
        output=True,
        frames_per_buffer=CHUNK,
    )

    audio_collector = []
    counter = 0
    unit = RATE // CHUNK
    text = ""
    start = time.time()
    print("start record...")
    while True:
        audio_data = stream.read(CHUNK)
        audio_collector.append(audio_data)
        counter += 1
        if counter % (int(unit * args.record_time) + 1) == 0:
            audio_collector = np.frombuffer(b"".join(audio_collector), dtype=np.int16)
            audio_collector = audio_collector.astype(np.float32) / INT16_MAX_ABS_VALUE
            print("record time: ", time.time() - start)
            # segments, info = stt_model.transcribe(
            #     audio_collector,
            #     language=args.record_src_language,
            #     beam_size=5,
            #     log_prob_threshold=-0.5,
            # )
            segments = pipe(audio_collector, generate_kwargs=generate_kwargs)

            text = ""
            # print("[%.2fs -> %.2fs] %s" % (segment.start, segment.end, segment.text))
            if segments['text'] != "":
                text += segments['text']
            # initial
            audio_collector = []
            counter = 0
            break

    # 关闭流和PyAudio对象
    stream.stop_stream()
    stream.close()
    p.terminate()

    return text


def main() -> None:
    global signal

    print("record_lang: ", args.record_src_language)

    if args.is_translate:
        tokenizer = AutoTokenizer.from_pretrained(args.translate_model)
        translate_model = AutoModelForSeq2SeqLM.from_pretrained(args.translate_model)

        translator = pipeline(
            "translation",
            model=translate_model,
            tokenizer=tokenizer,
            src_lang=args.translate_src_lang,
            tgt_lang=args.translate_tgt_lang,
            max_length=512,
            device=0,
        )
        print(
            "translate model is ready, src_lang: ",
            args.translate_src_lang,
            "tgt_lang: ",
            args.translate_tgt_lang,
        )

    while True:
        kb_listener()
        if signal == 1:
            break
        text = speech_to_text()
        print("record is done")
        translated_text = ""
        if args.is_translate and text != "":
            for tl in translator(text):
                translated_text += tl["translation_text"]
            print("sequence is translated")
            text = translated_text
        pyperclip.copy(text)


if __name__ == "__main__":
    main()
